#!/bin/bash
#
# Copyright (C) 2001-2002 Caldera International, Inc.  All Rights Reserved.
#

# turn on the log
PILOG=/var/tmp/postinstall.log; >$PILOG
UPGRADE_DIR=/var/tmp/msgupgrade

# are we in an upgrade situation
if [ -d $UPGRADE_DIR ] ; then
	UPGRADE=1
else
	UPGRADE=0
fi
UPGRADE_SAVEDIR=$UPGRADE_DIR/save

# Important files and directories
SCRIPTPATH=`dirname $0`
CONFIG=/etc/opt/lsb-sco.com/msg/config
MSGCONF=/etc/opt/lsb-sco.com/msg/msg.conf
SAVEDIR=/etc/opt/lsb-sco.com/msg/save
SAVEDIR_VAR=/var/opt/lsb-sco.com/msg/save
MSGBIN=/opt/lsb-sco.com/msg/bin
MSGETC=/etc/opt/lsb-sco.com/msg
MSGVAR=/var/opt/lsb-sco.com/msg
MSGTMP=$MSGVAR/tmp

# Discover installation platform
UNITEDLINUX=`$SCRIPTPATH/platform.sh --ul`;	
UNIXWARE=`$SCRIPTPATH/platform.sh --uw`

if [ $UNITEDLINUX -eq 1 ]; then
	MSG_INITPATH="/etc/init.d"
	MSG_LDAPDIR="/etc/openldap";
	CALDIR=/srv/ftp/pub/calendar
	HTHOME=/srv/www/htdocs
else
	MSG_INITPATH="/etc/rc.d/init.d"
	MSG_LDAPDIR="/etc/ldap";
	CALDIR=/home/ftp/pub/calendar
	HTHOME=/home/httpd/html
fi

export MSG_UNITEDLINUX=$UNITEDLINUX
export MSG_UPGRADE=$UPGRADE
export MSG_INITPATH MSG_LDAPDIR

# Enforce root privledge
USERNAME=`id -un`
if [ "$USERNAME" != "root" ] ; then
    echo " "
    echo "You must be root to run this script"
    exit 1
fi


##############################################################################
#
# VARIABLES
#
# Discover/set variables key to all the messaging components communicating
# with LDAP
#
##############################################################################

#
# Begin OL 310 Section
#
# This is only used for 3.1.0 systems
# NOTE: this DN must be "normalized" (no spaces around commas) for use in ACLs
#OL310_COUNTRY=US
#OL310_FQDN=`hostname --fqdn`; export OL310_FQDN
#if echo $OL310_FQDN | grep -q '.'
#then
#        OL310_ORG=`echo $OL310_FQDN | awk -F. '{print $(NF-1) "." $NF}'`
#else
#        OL310_ORG="$OL310_FQDN"
#fi
#OL310_ROOT="o=$OL310_ORG,c=$OL310_OUNTRY"
#OL310_ROOTDNBIND="uid=ldaproot,$OL310_ROOT"
#export OL310_COUNTRY OL310_ORG OL310_ROOT
#
# End OL 310 Section
#

#
# Can't do anything without a slapd.conf file
#
if [ ! -e "$MSG_LDAPDIR/slapd.conf" ]; then
	echo "Can not find an existing $MSG_LDAPDIR/slapd.conf file!  You must" >$2
	echo "not have openldap installed.  Exiting..." >$2
	exit 1
fi

#
# Take only the top and second level for the domain name
#
FQDN=`hostname --fqdn`
DOMAIN=`hostname --domain`
ROOT=`awk '/^suffix/ {print $2}' $MSG_LDAPDIR/slapd.conf | sed s/\"//g`
ROOTDNBIND=`awk '/^rootdn/ {print $2}' $MSG_LDAPDIR/slapd.conf | sed s/\"//g`
ROOTDNPW=`awk '/^rootpw/ {print $2}' $MSG_LDAPDIR/slapd.conf`
ROOTDNPWCRYPT=$ROOTDNPW

#
# If the ROOTDNPW is encrypted we need to generate and encrypt a new password.
#
if echo $ROOTDNPW | grep '^{' > /dev/null; then
	if [ -c /dev/random ]; then
		ROOTDNPW=`head -c16 /dev/random | uuencode -m - | sed -n '2s/=*$//;2p' | sed -e '1,$s/\//X/g'`; 
		ROOTDNPWCRYPT=`/usr/sbin/slappasswd -s $ROOTDNPW`
		while echo $ROOTDNPWCRYPT | grep '[=\/]' > /dev/null 2>&1
		do
			ROOTDNPW=`head -c16 /dev/random | uuencode -m - | sed -n '2s/=*$//;2p' | sed -e '1,$s/\//X/g'`; 
			ROOTDNPWCRYPT=`/usr/sbin/slappasswd -s $ROOTDNPW`
		done
	else 
		echo "ERROR: /dev/random not found!  Cannot generate ROOTDNPW..." >&2
		exit 1
	fi
fi

BASE="ou=SCOoffice Server,$ROOT"

#
# the bind DN for read access by postfix and pam_ldap
#
MSGUSER="msguser"
MSGUSERBIND="uid=$MSGUSER,$BASE"
if [ -c /dev/random ]; then
	MSGUSERPW=`head -c16 /dev/random | uuencode -m - | sed -n '2s/=*$//;2p' | sed -e '1,$s/\//X/g'`
else 
	# This code will never be reached, but for the sake of
	# consistency...
	echo "ERROR: /dev/random not found!  Cannot generate MSGUSERPW..." >&2
	exit 1
fi
MSGUSERPWCRYPT="{CRYPT}`$MSGBIN/msgencrypt $MSGUSERPW`"

# Free-busy password generation - only needed for UL
if [ "$UNITEDLINUX" -eq 1 ] 
then
	if [ -c /dev/random ]; then
		FREEBUSYPW=`head -c16 /dev/random | uuencode -m - | sed -n '2s/=*$//;2p' | sed -e '1,$s/\//X/g'`
	else 
		# This code will never be reached, but for the sake of
		# consistency...
		echo "ERROR: /dev/random not found!  Cannot generate FREEBUSYPW..." >&2
		exit 1
	fi
fi


#
# the bind DN for the MSG administrator
#
MSGADMIN="admin"
MSGADMINBIND="uid=$MSGADMIN,ou=MSG Users,ou=$DOMAIN,$BASE"
MSGADMINPW="admin"
MSGADMINPWCRYPT="{CRYPT}`$MSGBIN/msgencrypt $MSGADMINPW`"


###############################################################################
#
# OPENLDAP - backup databases
#
# Save the ldbm files if they aren't saved already
###############################################################################
function backup_ldap () {
	[ -d $SAVEDIR ] || { mkdir -p $SAVEDIR; }
	[ -d $SAVEDIR_VAR ] || { mkdir -p $SAVEDIR_VAR; }

	if [ $UNITEDLINUX -eq 1 ] ; then
		DIR="/var/lib/ldap";

		if [ -d $DIR ] ; then
			# We're going to switch the database format so get rid
			# of any old-format database files
			if ls $DIR/*.bdb >/dev/null 2>&1; then
				rm -f $DIR/*
			fi
		else
			mkdir -p -m 0700 $DIR
		fi
	else
		DIR="/var/ldap/openldap-ldbm";

		if [ -d $DIR ] ; then
			if [ ! -d $SAVEDIR_VAR/`basename $DIR` ] ; then
				mkdir -p $SAVEDIR_VAR/`basename $DIR`
				cwd=`pwd`;
				cd $DIR; find . | cpio -dump $SAVEDIR_VAR/`basename $DIR`
				cd $cwd
			fi
		else
			mkdir -p -m 0700 $DIR
		fi
	fi
}

###############################################################################
#
# POSTFIX - Configure postfix files
#
###############################################################################
function conf_postfix () {
	# Upgrade main.cf need to set the timeout to 60 for ldap
	# and add "check_relay_domains"
	postfix_upgrade=0;
	grep "SCOoffice Mail Server" /etc/postfix/main.cf >/dev/null 2>&1 && { postfix_upgrade=1; }
	if [ "$postfix_upgrade" -eq 1 ] ; then
		cat /etc/postfix/main.cf | \
			sed -e 's/ldapsource_timeout = 12/ldapsource_timeout = 60/' | \
			sed -e 's%^smtpd_recipient_restrictions.*$%smtpd_recipient_restrictions = check_client_access btree:/etc/postfix/dracd, permit_mynetworks, check_relay_domains%' | \
			sed -e 's/^alias_maps/#alias_maps/' > \
			/tmp/postfixmain.cf

		grep ^virtual_maps /tmp/postfixmain.cf > /dev/null || {
			echo "virtual_maps=ldap:ldapsource" >> /tmp/postfixmain.cf
 		}

		grep ^transport_maps /tmp/postfixmain.cf > /dev/null || {
			echo "transport_maps=hash:/etc/postfix/transport" >> /tmp/postfixmain.cf
 		}
	
		cp /tmp/postfixmain.cf /etc/postfix/main.cf
		rm /tmp/postfixmain.cf
	
	else
		cp $CONFIG/etc/postfix/main.cf /etc/postfix
	fi
	
	# set myhostname if not already set
	grep ^myhostname /etc/postfix/main.cf > /dev/null || {
        	cat <<END >>/etc/postfix/main.cf
myhostname = `hostname -f`
error_notice_recipient = postmaster@$FQDN
bounce_notice_recipient = \$error_notice_recipient
2bounce_notice_recipient = \$error_notice_recipient
delay_notice_recipient = \$error_notice_recipient
END
	}

	# Need to add the host info to the transport
	# map if its not already there.
	grep "^$DOMAIN[ 	]" /etc/postfix/transport > /dev/null || {
		cat <<END >>/etc/postfix/transport
$DOMAIN lmtp:unix:/var/imap/socket/lmtp
END
	}

	# If the FQDN and the DOMAIN don't match (which should
	# be, by far, the most common case) then set the real
	# fully qualified domain name for local delivery so the
	# aliases file and local accounts work.
	if [ $FQDN != $DOMAIN ]; then
	grep "^$FQDN[ 	]" /etc/postfix/transport > /dev/null || {
		cat <<END >>/etc/postfix/transport
$FQDN local:
END
	}
	fi

	# Build the transport map
	/usr/sbin/postmap /etc/postfix/transport

	# Make sure postfix has access to cryus LMTP
	chgrp postfix /var/imap
	chgrp postfix /var/imap/socket
	chmod 770 /var/imap
	chmod 770 /var/imap/socket
}

###############################################################################
#
# POSTFIX - aliases and maps
#
# re-compile the various postfix maps
#
###############################################################################
function conf_postfix_maps () {
	if [ $UNITEDLINUX -eq 1 ]; then
		/usr/sbin/postalias hash:/etc/aliases
	else
		/usr/sbin/postalias hash:/etc/postfix/aliases
	fi

	MAPS="access canonical relocated transport virtual"
	for f in $MAPS ; do
        	/usr/sbin/postmap hash:/etc/postfix/${f} >> $PILOG 2>&1
	done
}

###############################################################################
#
# MSGLDAPHOST
#
# fix up config files using the ldaphost utility.
# updates postfix, ldap.conf, msg.conf
###############################################################################
function conf_msgldaphost () {
	$MSGBIN/msgldaphost \
		--base="$BASE" \
		--rootdn="$ROOTDNBIND" \
		--admin="$MSGADMIN" \
		--admindn="$MSGADMINBIND" \
		--uid="$MSGUSER" \
		--dn="$MSGUSERBIND" \
		--pw="$MSGUSERPW" \
		>> $PILOG 2>&1
}

###############################################################################
#
# SLAPDCONF
#
# now configure slapd
# make the slapd.conf file readable by postfix and cyrus
# maintain ownership of slapd.conf to original owner
# make sure the ldap database directory is there.
#
###############################################################################
function conf_slapdconf () {
	cp $MSG_LDAPDIR/slapd.conf $SAVEDIR
	$SCRIPTPATH/slapdconf \
        	--base="$BASE" \
        	--root="$ROOT" \
        	--rootdn="$ROOTDNBIND" \
        	--rootdnpw="$ROOTDNPWCRYPT" \
        	--conffile=$MSG_LDAPDIR/slapd.conf >> $PILOG 2>&1
	chmod 640 $MSG_LDAPDIR/slapd.conf
	OLDOWNER=$(ls -l $SAVEDIR/slapd.conf | awk '{print $3 ":" $4}')
	chown $OLDOWNER $MSG_LDAPDIR/slapd.conf >> $PILOG 2>&1
}


###############################################################################
#
# IMAP
#
# now configure imapd
# make sure imapd.conf is properly readable
# make sure that postfix and cyrus users can resolve addresses
#
###############################################################################
function conf_imap () {
	# Turn on singleinstance message store
	grep ^singleinstancestore /etc/imapd.conf > /dev/null || {
        	cat <<END >>/etc/imapd.conf
singleinstancestore: yes
END
	}

	sed -e "s/%ADMIN%/$MSGADMIN/" /etc/imapd.conf > /var/tmp/imapd.conf
	mv -f /var/tmp/imapd.conf /etc/imapd.conf
	chmod 644 /etc/imapd.conf
	chmod 644 /etc/resolv.conf
}
function conf_public_folders () {
	# Create the All Public Folders name space
	printf "$MSGADMINBIND\n$MSGADMINPW" | \
		$MSGBIN/msgpubcreate --name='"All Public Folders"'
}


###############################################################################
#
# SSL - Certificates
#
# Add certificates to the system
#
###############################################################################
function conf_certs () {
	$SCRIPTPATH/installcerts
}

###############################################################################
#
# MOD_AUTH_LDAP
#
# fix up the ldap config to increase the timeout value
#
###############################################################################
function conf_modauthldap () {
	file=/etc/pam_ldap/auth_ldap.conf
	tmp=/tmp/ldap$$

	if [ -f $file ]; then
		awk '
	/^bind_timelimit/ { next }
	/.*/ { print $0; next }
	END { print "bind_timelimit 60" }
	' < $file > $tmp
	
		cp $tmp $file
		rm -f $tmp
	fi
	unset file
	unset tmp
}


###############################################################################
#
# DOCVIEW
#
# Need to fix OpenLinux 3.1.1 docview so that it serves
# more than just the local machine.
###############################################################################
function conf_docview () {
	DOCCONF="/usr/lib/docview/conf/httpd.conf"
	if [ -f $DOCCONF ] ; then

        	sed "1,\$s/^Listen .*/Listen `hostname`:8457/" $DOCCONF > \
			$DOCCONF.1

        	sed "1,\$s/^ServerName .*/ServerName `hostname`/" $DOCCONF.1 > \
			$DOCCONF.2

        	sed "1,\$s/^BindAddress/#BindAddress/" $DOCCONF.2 > \
			$DOCCONF.new
	 
        	rm -f $DOCCONF.1 $DOCCONF.2
        	cp -f $DOCCONF.new $DOCCONF

	fi
}


###############################################################################
#
# RESTART PROCESSES
#
# start everything up
#  1. ldap
#  2. drac
#  3. imap
#  4. postfix
#  5. saslauthd
#
#  1. apache (done below, after apache config)
#  2. docview (done below, after apache config)
#
###############################################################################
function proc_start () {
	cwd=`pwd`
	cd /var/run
	[ -f $MSG_INITPATH/ldap    ] && $MSG_INITPATH/ldap    start
	[ -f $MSG_INITPATH/dracd   ] && $MSG_INITPATH/dracd   start
	[ -f $MSG_INITPATH/cyrus   ] && $MSG_INITPATH/cyrus   start
	[ -f $MSG_INITPATH/postfix ] && $MSG_INITPATH/postfix start
	[ -f $MSG_INITPATH/saslauthd ] && $MSG_INITPATH/saslauthd start
	cd $cwd

	sleep 5
}

function proc_start2 () {
	cwd=`pwd`
	cd /var/run

	[ -f /usr/sbin/apachectl ] && {
		/usr/sbin/apachectl stop
		sleep 5
		/usr/sbin/apachectl startssl
	}

	[ -f $MSG_INITPATH/docview ] && {
		$MSG_INITPATH/docview stop
		sleep 5
		$MSG_INITPATH/docview start
	}

	if [ $UNITEDLINUX -eq 1 ] ; then
		/etc/init.d/inetd start
	fi
	cd $cwd
}

function proc_stop() {
	[ -f $MSG_INITPATH/ldap    ] && $MSG_INITPATH/ldap stop >/dev/null 2>&1
	[ -f $MSG_INITPATH/dracd   ] && $MSG_INITPATH/dracd stop >/dev/null 2>&1
	[ -f $MSG_INITPATH/cyrus   ] && $MSG_INITPATH/cyrus stop >/dev/null 2>&1
	[ -f $MSG_INITPATH/postfix ] && $MSG_INITPATH/postfix stop >/dev/null 2>&1
	[ -f $MSG_INITPATH/saslauthd ] && $MSG_INITPATH/saslauthd stop >/dev/null 2>&1
	[ -f $MSG_INITPATH/docview ] && $MSG_INITPATH/docview stop >/dev/null 2>&1
	[ -f $MSG_INITPATH/apache ] && $MSG_INITPATH/apache stop >/dev/null 2>&1

	#
	# Make sure ldap is really, really stopped
	#
	killall -15 slapd >/dev/null 2>&1
	sleep 5
	killall -9 slapd >/dev/null 2>&1
	sleep 5
	[ -e /var/run/slapd/slapd.pid ] && { rm -f /var/run/slapd/slapd.pid; }
	[ -e /var/run/slapd.pid ] && { rm -f /var/run/slapd.pid; }
}



###############################################################################
#
# POPULATE LDAP
#
# Initialize the LDAP database
# FIXME - If this has already been done, don't do it again!  See how to
# check...
#
###############################################################################
function conf_ldap () {
	$SCRIPTPATH/initldapdb \
        	--root="$ROOT" \
        	--base="$BASE" \
        	--fqdn="$DOMAIN" \
        	--rootdn="$ROOTDNBIND" \
        	--rootdnpw="$ROOTDNPW" \
        	--admindn="$MSGADMINBIND" \
        	--dn="$MSGUSERBIND" \
        	--uid="$MSGUSER" \
        	--userpw="$MSGUSERPWCRYPT" \
	        --admin="$MSGADMIN" \
        	--adminpw="$MSGADMINPWCRYPT" \
        	--ldiffile=$MSGETC/ldif.base >> $PILOG 2>&1
}

###############################################################################
#
# ADMIN ACCOUNT
#
# create an inbox for $MSGADMIN
#
###############################################################################
function conf_adminaccount () {
	printf "$MSGADMINBIND\n$MSGADMINPW" | \
		$MSGBIN/msginboxcreate --name=$MSGADMIN
}


###############################################################################
#
# APACHE/PHP/FTP - gui, ftp free-busy
#
# Set up apache for the webgui and dav (free-busy)
# Set PHP to register_globals
# Set up FTP and the webserver for calendar support
#
###############################################################################
function conf_apache () {
	[ -d $CALDIR ] || {
		mkdir -p $CALDIR
		chown ftp:ftp $CALDIR
		chmod 755 $CALDIR
		[ -L $HTHOME/calendar ] || {
			ln -s $CALDIR $HTHOME/calendar
		}
	
		if [ $UNITEDLINUX -ne 1 ] ; then
        	cat <<END >/etc/inet.d/wu-ftpd
ftp    stream  tcp     nowait  root    /usr/sbin/tcpd  in.ftpd -l -a
END
		killall -HUP inetd
		fi
		if [ "$UNITEDLINUX" -eq 1 ] ; then
			FTPUID=`grep ^ftp /etc/passwd | awk -F':' ' { print $3 } '`
			if [ -z "$FTPUID" ]; then
				echo "ERROR: ftp account not found" >&2
				echo "ERROR: freebusy account not added" >&2
			else
				useradd -u $FTPUID -s /bin/false -o -d /srv/ftp/ freebusy
				cat <<END >/var/tmp/freebusy
freebusy:$FREEBUSYPW
END
				chpasswd < /var/tmp/freebusy
				rm -f /var/tmp/freebusy
				perl -pi -e "s/:.*$/:$FREEBUSYPW/ if (/^FREEBUSYPW:/);" $MSGCONF
			fi
		fi
	}
	$SCRIPTPATH/apachecfg.pl
	if [ $UNITEDLINUX -eq 1 ]; then
		perl -pi -e 's/=\s*Off/= On/i if (/register_globals\s*=\s*Off/i);' /etc/php.ini
		perl -pi -e 's/Group.*$/Group www/i if (/^Group /i);' /etc/httpd/httpd.conf
		perl -pi -e 's/^#\sftp/ftp/i if (/vsftpd$/i);' /etc/inetd.conf
	fi

}

function upg_apache () {
	$SCRIPTPATH/apachecfg.pl
}

###############################################################################
#
# DAEMONS
#
# This is for OpenLinux Platforms
# Stop/kill hotplugd and set it to never start again!
# Make slapd and httpd start upon reboot
# LKP does not enable docview for startup at install time
#
###############################################################################
function conf_daemons () {
	if [ -e /etc/sysconfig/daemons/hotplugd ]; then
        	sed -e 's/ONBOOT=yes/ONBOOT=no/g' \
			/etc/sysconfig/daemons/hotplugd > /tmp/hotplugd
        	mv /tmp/hotplugd /etc/sysconfig/daemons
        	/etc/rc.d/init.d/hotplugd stop
	fi

	[ -f /etc/sysconfig/daemons/ldap ] && {
		sed -e 's/^ONBOOT=.*/ONBOOT="yes"/' \
			/etc/sysconfig/daemons/ldap > /var/tmp/ldap
		mv -f /var/tmp/ldap /etc/sysconfig/daemons/ldap
		chmod 644 /etc/sysconfig/daemons/ldap
		chown root:root /etc/sysconfig/daemons/ldap
	}

	[ -f /etc/sysconfig/daemons/httpd ] && {
		sed -e 's/^ONBOOT=.*/ONBOOT="yes"/' \
			/etc/sysconfig/daemons/httpd > /var/tmp/httpd1
		sed -e 's/^OPTIONS=.*/OPTIONS="-DSSL"/' \						/var/tmp/httpd1 > /var/tmp/httpd2
		rm -f /var/tmp/httpd1
		mv -f /var/tmp/httpd2 /etc/sysconfig/daemons/httpd
		chmod 644 /etc/sysconfig/daemons/httpd
		chown root:root /etc/sysconfig/daemons/httpd
	}

	[ -f /etc/sysconfig/daemons/docview ] && {
		sed -e 's/^ONBOOT=.*/ONBOOT="yes"/' \
			/etc/sysconfig/daemons/docview > /var/tmp/docview
		mv -f /var/tmp/docview /etc/sysconfig/daemons/docview
		chmod 644 /etc/sysconfig/daemons/docview
		chown root:root /etc/sysconfig/daemons/docview
	}

	if [ -f /etc/sysconfig/daemons/nscd ]; then
        	sed -e 's/ONBOOT=yes/ONBOOT=no/g' /etc/sysconfig/daemons/nscd > /tmp/nscd
        	mv /tmp/nscd /etc/sysconfig/daemons
        	/etc/rc.d/init.d/nscd stop
	fi

	# and one UnitedLinux daemon
	if [ $UNITEDLINUX -eq 1 ] ; then
		# ensure that openldap is configured to start on system boot
		insserv /etc/init.d/ldap 
	fi
}

###############################################################################
#
# SIEVEBUILD
#
# add sievebuild to cron
#
###############################################################################
function conf_sievebuild () {

	crontab -u root -l 2>/dev/null | sed -e /sievebuild/d >/tmp/crontab$$
 
	if [ $UNIXWARE -eq 1 ] ; then
		build="17,47 * * * * (cd /linux/bin; ./bash -c $MSGBIN/msgsievebuild)"
	else
		build="17,47 * * * * $MSGBIN/msgsievebuild"
	fi
 
	cat <<END >/tmp/crontmp$$
$build
END
 
	crontab -u root /tmp/crontmp$$
	rm -f /tmp/crontmp$$
	unset build
}

###############################################################################
#
# HORDE/IMP
#
# configure webmail/mysql
#
###############################################################################
function conf_hordeimp () {
	$MSGBIN/msgimpsetup
}

function conf_msgconf_guiids () {
	chgrp root $MSGBIN/msgexec
	if [ $UNITEDLINUX -eq 1 ]; then
		perl -pi -e 's/:.*$/:wwwrun/i if (/^GUIUID:/i);' $MSGCONF
		perl -pi -e 's/:.*$/:www/i if (/^GUIGID:/i);' $MSGCONF
		chgrp www $MSGBIN/msgexec
	else
		perl -pi -e 's/:.*$/:httpd/i if (/^GUIUID:/i);' $MSGCONF
		perl -pi -e 's/:.*$/:httpd/i if (/^GUIGID:/i);' $MSGCONF
		chgrp httpd $MSGBIN/msgexec
	fi
	touch $MSGBIN/msgexec
}

function conf_msgconf () {
	if [ $UNITEDLINUX -eq 1 ]; then
		/bin/chgrp www $MSGCONF
	fi
	conf_msgconf_guiids
}

function upg_msgconf () {
	echo "Upgrading msg.conf ... "

	savefile="$UPGRADE_SAVEDIR/etc/opt/lsb-sco.com/msg/msg.conf"
	attributes='LDAPHOST LDAPSUFFIX ROOTDN ROOTPW ADMIN ADMINDN DN
		USER PW SECURE OUTLKMBOXTYPE'
	for i in $attributes ; do
		val=`grep "^$i:" $savefile | head -1`
		perl -pi -e "s/^.*$/$val/ if (/^$i:/);" $MSGCONF
		unset val
	done
	unset savefile

	if [ $UNITEDLINUX -eq 1 ]; then
		/bin/chgrp www $MSGCONF
	fi
	conf_msgconf_guiids
}

function upg_imapdconf () {
	echo "Upgrading imapd.conf ... "

	admin=`grep "^ADMIN:" $MSGCONF | head -1 | sed -e "s/ADMIN://"`
	perl -pi -e "s/%ADMIN%/$admin/ if (/^admins/);" /etc/imapd.conf
	perl -pi -e "s/:.*$/: no/ if (/^altnamespace:/);" /etc/imapd.conf
	unset admin
}

function upg_slapdconf () {
	echo "Upgrading slapd.conf ... "

	rootpw=`grep "^rootpw" $UPGRADE_SAVEDIR/$MSG_LDAPDIR/slapd.conf | head -1`
	base=`grep "^LDAPSUFFIX:" $MSGCONF | head -1 | sed -e "s/LDAPSUFFIX://"`
	cp $MSG_LDAPDIR/slapd.conf.default $MSG_LDAPDIR/slapd.conf
	$SCRIPTPATH/ldschema.pl
        cp /var/tmp/slapd.conf $MSG_LDAPDIR/slapd.conf
	perl -pi -e "s/^.*$/$rootpw/ if (/^rootpw\s/);" $MSG_LDAPDIR/slapd.conf
	perl -pi -e "s/%BASE%/$base/;" $MSG_LDAPDIR/slapd.conf

	unset rootpw base
}

function upg_ldapconf () {
	echo "Upgrading ldap.conf ... "

	cp $CONFIG/$MSG_LDAPDIR/ldap.conf $MSG_LDAPDIR/ldap.conf

	base=`grep "^LDAPSUFFIX:" $MSGCONF | head -1 | sed -e "s/LDAPSUFFIX://"`
	perl -pi -e "s/%BASE%/$base/ if (/^BASE/);" $MSG_LDAPDIR/ldap.conf
	unset base
}

function upg_authldapconf () {
	echo "Upgrading auth_ldap.conf ... "

	cp $UPGRADE_SAVEDIR/etc/pam_ldap/auth_ldap.conf \
		/etc/pam_ldap/auth_ldap.conf

	perl -pi -e "s/Volution Messaging Server/SCOoffice Server/;" \
		/etc/pam_ldap/auth_ldap.conf
	perl -pi -e "s/\son/ off/i if (/^ssl /);" \
		/etc/pam_ldap/auth_ldap.conf
}

function upg_postfix () {
	echo "Upgrading postfix ... "

	cp $UPGRADE_SAVEDIR/etc/postfix/main.cf /etc/postfix/main.cf
 
	base=`grep "^LDAPSUFFIX:" $MSGCONF | head -1 | sed -e "s/LDAPSUFFIX://"`
	dn=`grep "^DN:" $MSGCONF | head -1 | sed -e "s/DN://"`
        perl -pi -e "s/=.*$/= $base/ if (/^ldapsource_search_base.*/);" \
                /etc/postfix/main.cf
        perl -pi -e "s/=.*$/= $dn/ if (/^ldapsource_bind_dn.*/);" \
                /etc/postfix/main.cf
        perl -pi -e "s/^/# / if (/^\s*ldapsource_domain/);" \
                /etc/postfix/main.cf
        unset base dn
 
        if ! grep "^virtual_maps" /etc/postfix/main.cf ; then
                echo "virtual_maps = " >> /etc/postfix/main.cf
        fi
 
        if ! grep "^transport_maps" /etc/postfix/main.cf ; then
                echo "transport_maps = " >> /etc/postfix/main.cf
        fi
 
        perl -ni -e '
                if (/^local_recipient_maps.*/) {
                        print "local_recipient_maps = \$virtual_maps\n";
                } elsif (/^virtual_maps.*/) {
                        print "virtual_maps = ldap:ldapsource\n";
                } elsif (/^transport_maps.*/) {
                        print "transport_maps = hash:/etc/postfix/transport\n";
                } elsif (/^ldapsource_query_filter.*/) {
                        print "ldapsource_query_filter = (&(|(objectclass=msgUser)(objectclass=msgGroup))(|(mail=%s)(calderaMsgAlternateAddress=%s)))\n";
                } elsif (/^ldapsource_result_attribute.*/) {
                        print "ldapsource_result_attribute = calderaMsgDrop, calderaMsgForwardingAddress, calderaMsgMgrpRFC822Member\n";
                } elsif (/^alias_maps.*/) {
                        print "alias_maps = hash:/etc/postfix/aliases\n";
                } else {
                        print $_;
                }
        ' /etc/postfix/main.cf

	# Need to add the host info to the transport
	# map if its not already there.
	for i in `cat $UPGRADE_DIR/domains` ; do
		grep "^$i[ 	]" /etc/postfix/transport >/dev/null || {
			cat <<END >>/etc/postfix/transport
$i lmtp:unix:/var/imap/socket/lmtp
END
		}
	done

	# If the FQDN and the DOMAIN don't match (which should
	# be, by far, the most common case) then set the real
	# fully qualified domain name for local delivery so the
	# aliases file and local accounts work.
	if [ $FQDN != $DOMAIN ]; then
	grep "^$FQDN[ 	]" /etc/postfix/transport > /dev/null || {
		cat <<END >>/etc/postfix/transport
$FQDN local:
END
	}
	fi

	# Fix up the master.cf file
	echo "Restoring master.cf... "
	cp $UPGRADE_SAVEDIR/etc/postfix/master.cf /etc/postfix/master.cf
        perl -pi -e "
		my \$old = 'lsb-caldera.com-volution/msg';
		my \$new = 'lsb-sco.com/msg';
		s/\$old/\$new/;" /etc/postfix/master.cf

	# Make sure postfix has access to cryus LMTP
	chgrp postfix /var/imap
	chgrp postfix /var/imap/socket
	chmod 770 /var/imap
	chmod 770 /var/imap/socket
}

function upg_files () {

	oldfiles='etc/opt/lsb-sco.com/msg/vscan.conf'
	for i in $oldfiles ; do
		echo "Restoring `basename $i` ... "
		cp $UPGRADE_SAVEDIR/$i /$i
	done

	#etc/ldap/slapd.conf - subsitutions and old?
	#echo "Upgrading ... "
	#newfiles=''
	#for i in $newfiles ; do
	#	cp $CONFIG/$i /$i
	#done
}

function upg_hordeimp () {

	# OpenLinux only
	if [ $UNITEDLINUX -ne 1 ]; then
		# update the product specific information 
		perl -pi -e "s/volutionmsg/scomsg/;" \
		    $HTHOME/horde/imp/config/ldap.php3
		perl -pi -e \
		    "s/ou=Volution Messaging Server/ou=SCOoffice Server/;" \
		    $HTHOME/horde/imp/config/ldap.php3
		perl -pi -e "s/Volution Messaging Server/SCOoffice Mail Server/;" \
		    $HTHOME/horde/imp/config/ldap.php3

		# this fixes an error in how 1.1.1 configured these entries
 		domain=`grep '^$default->from_server' $HTHOME/horde/imp/config/defaults.php3 | sed -e "s/^.*'\(.*\)'.*$/\1/"`
		perl -pi -e "s/'root\@.*'/'root\@$domain'/ if (/^\\\$default->problem_email/);" $HTHOME/horde/config/horde.php3
		perl -pi -e "s/'root\@.*'/'root\@$domain'/ if (/^\\\$default->signup_email/);" $HTHOME/horde/config/horde.php3

	fi
}




###############################################################################
#
# ----------------------------   M A I N  ------------------------------------
#
###############################################################################

if [ $UPGRADE -eq 1 ] ; then

	proc_stop

	upg_apache
	upg_msgconf
	upg_slapdconf
	upg_imapdconf
	upg_ldapconf
	upg_authldapconf
	upg_postfix
	upg_files

	proc_start
	proc_start2

	$MSGBIN/msgipu

	upg_hordeimp

else 
	proc_stop

	backup_ldap
	conf_postfix
	conf_msgldaphost
	conf_slapdconf
	conf_imap
	conf_certs
	conf_postfix_maps
	conf_modauthldap
	conf_docview

	proc_start

	conf_ldap
	conf_adminaccount
	conf_public_folders
	conf_apache
	conf_daemons

	proc_start2

	conf_sievebuild
	conf_hordeimp
	conf_msgconf

fi
