#!/bin/sh
#
# Copyright (C) 2002 Caldera International, Inc.  All Rights Reserved.
#

#
# This file does was written to help upgrade VMS 1.1.1 to
# SCOoffice Mail Server 2.0.0 on OpenLinux platform
#
SCRIPTPATH=`dirname $0`

UNITEDLINUX=`$SCRIPTPATH/platform.sh --ul`;	
if [ $UNITEDLINUX -eq 1 ]; then
	MSG_LDAPDIR="/etc/openldap";
else
	MSG_LDAPDIR="/etc/ldap";
fi

savedir='/var/tmp/msgupgrade/save'
savefiles='/etc/ldap/slapd.conf
/etc/ldap/ldap.conf
/etc/ldap/schema/msg.schema
/etc/ldap.conf
/etc/imapd.conf
/etc/postfix/main.cf
/etc/postfix/master.cf
/etc/pam_ldap/auth_ldap.conf
/etc/opt/lsb-caldera.com-volution/msg/vscan.conf'

savefiles2='/etc/opt/lsb-caldera.com-volution/msg/msg.conf
/etc/ldap/slapd.conf'

# let other scripts know that we are performing an upgrade
mkdir -p /var/tmp/msgupgrade

# backup files for later use
for i in $savefiles ; do
	mkdir -p $savedir/`dirname $i`
	cp -f $i $savedir/$i
done

# get rootdn and a rootpw that we can use in upgrade
ROOTDN=`awk '/^rootdn/ {print $2}' $MSG_LDAPDIR/slapd.conf | sed s/\"//g`
ROOTPW=`awk '/^rootpw/ {print $2}' $MSG_LDAPDIR/slapd.conf`
ROOTPWCRYPT=$ROOTPW
OLDROOTPW=$ROOTPW
# If the ROOTPW is encrypted we need to generate and encrypt a new password.
if echo $ROOTPW | grep '^{' > /dev/null; then
	if [ -c /dev/random ]; then
		ROOTPW=`head -c16 /dev/random | uuencode -m - | sed -n '2s/=*$//;2p' | sed -e '1,$s/\//X/g'`; 
		ROOTPWCRYPT=`/usr/sbin/slappasswd -s $ROOTPW`
		while echo $ROOTPWCRYPT | grep '[=\/]' > /dev/null 2>&1
		do
			ROOTPW=`head -c16 /dev/random | uuencode -m - | sed -n '2s/=*$//;2p' | sed -e '1,$s/\//X/g'`; 
			ROOTPWCRYPT=`/usr/sbin/slappasswd -s $ROOTPW`
		done
	else 
		echo "ERROR: /dev/random not found!  Cannot generate ROOTPW..." >&2
		exit 1
	fi
fi

# if we set a new password
if [ "$ROOTPW" != "$ROOTPWCRYPT" ] ; then
	perl -pi -e "s/^rootpw.*$/rootpw\t\t$ROOTPWCRYPT/ if (/^rootpw/);" $MSG_LDAPDIR/slapd.conf
fi

# need this file to restart ldap server
cp $savedir/etc/ldap/schema/msg.schema /etc/ldap/schema/msg.schema

# upgrade the ldap database
$SCRIPTPATH/upgrade.pl "$ROOTDN" "$ROOTPW" "$SCRIPTPATH"

# backup more files for later use
for i in $savefiles2 ; do
	mkdir -p $savedir/`dirname $i`
	cp -f $i $savedir/$i
done

# remove the old rpms
rpm -q volutionmsg-doc && { rpm -e volutionmsg-doc; }
rpm -q volutionmsg && { rpm -e volutionmsg; }

# remove specific config files if they are still around
rm -f /etc/opt/lsb-caldera.com/msg/msg.conf
rm -f /etc/opt/lsb-caldera.com/msg/allowed
rm -rf /opt/lsb-caldera.com-volution
rm -rf /etc/opt/lsb-caldera.com-volution

# update httpd.conf markers with the new name so that apachecfg.pl will see it
perl -pi -e 's/VOLUTION MESSAGING SERVER/SCOoffice Mail Server/i if (/VOLUTION MESSAGING SERVER/i);' /etc/httpd/httpd.conf

# update main.cf so that postinstall will recognize the upgrade
perl -pi -e 's/Volution Messaging Server/SCOoffice Mail Server/i if (/Volution/i);' /etc/postfix/main.cf

# rename remaining directories
from=lsb-caldera.com-volution
to=lsb-sco.com
for i in /var/opt $savedir/opt $savedir/etc/opt $savedir/var/opt
do
	if [ ! -d $i/$to ] ; then
		if [ -d $i/$from ] ; then
			mv -f $i/$from $i/$to
		fi
	fi
done

# make sure the old password is kept
perl -pi -e "s/^rootpw.*$/rootpw\t\t$OLDROOTPW/ if (/^rootpw/);" \
	$savedir/$MSG_LDAPDIR/slapd.conf
