<?php
/*
 * Copyright (C) 2002 Caldera International, Inc.  All Rights Reserved.
 */

class License {
	var $serial;	  // sv, license serial number (unique)
	var $description; // sv, descriptive name of license
	var $users;	  // sv, number of users enabled by license
	var $version;	  // sv, the license version

	/* Methods */

	function License() {
		$numArgs = func_num_args();

		switch ($numArgs) {
		  case 0:
			$this->_emptyLicense();
			break;
		  case 1:
			$arg1 = func_get_arg(0);
			if (is_array($arg1)) {
				$ret = $this->_addLicense($arg1);
				if (! $ret) {
					$this = '';
				}
			} else {
				$this->_fetchLicense($arg1);
			}
			break;
		  default:
			$this = '';
			break;
		}

	}

	function _emptyLicense() {
		; // nothing
	}

	function _fetchLicense($serial) {
		$cmd = new Command('msglicenselistall');
		$cmd->run();
		$lines = $cmd->getOutput();

		unset($this->list);

		// Parse the output looking for requested serial number
		$found = false;
		if ($lines != '') foreach ($lines as $line) {

			$line = trim($line);

			if ($line == '') {
				if ($found) {
					break;
				} else {
					continue;
				}
			}

			list($attr, $value) = split(':', $line, 2);

			if ($attr == 'serialNumber' && $value == $serial) {
				$found = true;
			}

			if ($found) {
				$this->setAttr($attr, $value);
			}
		}

		if (! $found) {
			$this = '';
		}
	}

	function _addLicense($varArray) {

		$cmd = new Command('msglicense');
		$cmd->add_arg("--add");

		while (list ($attr, $val) = each ($varArray)) {

			$val = trim($val);

			if ($val == '')  continue;

			$attr = str_replace('_', '', $attr);

			if (! $this->_validate($attr, $val)) continue;

			switch ($attr) {
				case 'actkey' :
				    $actkey = strtolower($val);
				    break;
				case 'data' :
				    $data = strtolower($val);
				    break;
				case 'serial':
				    $this->serial = $val;
				    break;
				default:
				    break;
			}

		}

		// If these aren't set or are invalid
		// let the back end provide the error
		$cmd->add_arg("--serialNumber", $this->serial);
		$cmd->add_arg("--activationKey", $actkey);
		if ($data != "") $cmd->add_arg("--licenseData", '\\\""' . $data . '"\\\"');

		$cmd->run();

		if ($cmd->getExitVal() == 0) {
			$this->_fetchLicense($this->serial);
			return true;
		}
		$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
		$GLOBALS['LAST_ERROR'][] = getIntlMsg('ERR_CMD_ON', $cmd->concat());
		$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
		return false;
	}

	function delete() {

		$cmd = new Command('msglicense');
		$cmd->add_arg("--remove");

		// If these aren't set or are invalid
		// let the back end provide the error
		$cmd->add_arg("--serialNumber", $this->serial);

		$cmd->run();

		if ($cmd->getExitVal() == 0) {
			$this='';
			return true;
		}
		$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
		$GLOBALS['LAST_ERROR'][] = getIntlMsg('ERR_CMD_ON', $cmd->concat());
		$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
		return false;
	}

	function setAttr($attr, $val) {
		switch ($attr) {
			case 'serialNumber':
				$this->serial = $val;
				break;
			case 'licenseName':
				$this->description = $val;
				break;
			case 'licenseUsers':
				$this->users = $val;
				break;
			case 'licenseVersion':
				$this->version = $val;
				break;
			default:
				break;
		}
	}

	function _validate() {
		return true;
	}

	function getSerial() {
		return $this->serial;
	}

	function getDescription() {
		return $this->description;
	}

	function getUsers() {
		return $this->users;
	}

	function getVersion() {
		return $this->version;
	}

	function comparisonFunctionName($field, $direction) {
		switch($field) {
			case 'serial':
				$cmpFunction = '_license_bySerial';
				break;
			case 'description':
				$cmpFunction = '_license_byDescription';
				break;
			case 'users':
				$cmpFunction = '_license_byUsers';
				break;
			default:
				$cmpFunction = '_license_bySerial';
				break;
		}

		switch($direction) {
			case 'a':
				$cmpFunction .= 'Ascend';
				break;
			case 'd':
				$cmpFunction .= 'Decend';
				break;
			default:
				$cmpFunction .= 'Ascend';
				break;
		}
		return $cmpFunction;
	}

	function isRequired($attr) {
		switch ($attr) {
			case 'serial':
			case 'actkey':
				$retval = true;
				break;
			case 'data':
			default:
				$retval = false;
				break;
		}
		return $retval;
	}
}

/*
 * Ideally these comparison functions would be encapsulated in the 
 * User class as class methods. Can't work out how to pass a class
 * method to the usort() function yet.
 */
function _license_bySerialAscend($left, $right) {
	return(strcasecmp($left->serial, $right->serial));
}

function _license_bySerialDecend($left, $right) {
	return(-1 * (_license_bySerialAscend($left, $right)));
}

function _license_byDescriptionAscend($left, $right) {
	return(strcasecmp($left->description, $right->description));
}

function _license_byDescriptionDecend($left, $right) {
	return(-1 * (_license_byDescriptionAscend($left, $right)));
}

function _license_byUsersAscend($left, $right) {
	if ($left->users < $right->users) {
		return -1;
	} elseif ($left->users > $right->users) {
		return 1;
	} else {
		// must be ==
		return 0;
	}
}

function _license_byUsersDecend($left, $right) {
	return(-1 * (_license_byUsersAscend($left, $right)));
}

class MasterLicense {
	var $serialNumber;
	var $licensePeriod;
	var $licenseRemaining;
	var $licenseUsers;
	var $usersCount;

	function MasterLicense() {
		$cmd = new Command("--licensePrint");
		$cmd->setAbsolutePath();
		$cmd->run();

		$lines = $cmd->getOutput();

		if ($lines != '') foreach ($lines as $line) {
			$line = trim($line);

			if ($line != '') {
				list($attr, $value) = split(':', $line, 2);
				switch ($attr) {
					case 'License' :
						$this->serialNumber = $value;
						break;
					case 'LicenseDays' :
						$this->licensePeriod = $value;
						break;
					case 'LicenseDaysRemaining' :
						$this->licenseRemaining = $value;
						break;
					case 'UserLicenses' :
						$this->licenseUsers = $value;
						break;
					case 'UserLicensesUsed' :
						$this->userCount = $value;
						break;
					default:
						break;
				}
			}
		}
		return false;
	}

	function isEval() {
		return ($this->serialNumber == 'EVAL') ? true : false;
	}

	function getDaysLeft() {
		if ($this->isEval()) {
			return $this->licenseRemaining;
		}
	}

	function noLicense() {
		return ($this->serialNumber == '') ? true : false;
	}

	function getLicensedUsers() {
		if ($this->isEval()) {
			return getIntlMsg('MSG_LICENCE_USERS_UNLIMITED');
		}
		return $this->licenseUsers;
	}

	function getUsersInUse() {
		return $this->userCount;
	}

	function getSerialNumber() {
		return $this->serialNumber;
	}

	function tooManyUsers() {
		if ($this->isEval()) {
			return false;
		}

		return ($this->userCount > $this->licenseUsers) ? true : false;
	}
}

?>
