<?php
/*
 * Copyright (C) 2001-2002 Caldera International, Inc.  All Rights Reserved.
 */

class User {
	var $errorMsgs;

	/* Properties */
	// sv = single valued, mv = multivalued
	var $uid; 	// sv, User Id (inbox name), *required*
	var $firstname;	// sv, First Name
	var $lastname;	// sv, Surname, *required*
	var $fullname;	// sv, Common Name (usually firstname + lastname)
	var $mail;	// sv, Email address (with domain), *required*
	var $shortmail;	// sv, Email address (without domain), *required*
	var $pw;	// sv, Password, *required*
	var $wphone;	// sv, Work phone number
	var $mphone;	// sv, Mobile phone number
	var $hphone;	// sv, Home phone number
	var $pager;	// sv, Pager number
	var $fax;	// sv, Fax number
	var $title;	// sv, Business Title
	var $location;  // sv, Physical office delievery location
	var $altmail;	// sv, Alternate email address
	var $forward;	// sv, Email address to forward mail to
	var $forwardcopy; // sv, boolean keep a copy of forwarded messages
	var $vacation;	// sv, boolean vacation notification active or not
	var $vsubject;	// sv, Subject line for vacation notification
	var $vmessage;	// sv, Message body for vacation notification
	var $spamfilter; // sv, boolean spam filtering active or not
	var $spamdestroy; // sv, boolean spam filtering destroys or saves mail
	var $spamallow; // mv, array of sub-strings of email addresses to allow
	var $spamdeny;  // mv, array of sub-strings of email addresses to reject
	var $quota;     // sv, quota for user in Kb or NONE
	var $quotaused; // sv, quota used for use in Kb or not set

	/* Methods */

	function User() {
		global $MSG_ID, $MSG_NAME, $MSG_STATUS;

		$numArgs = func_num_args();

		switch ($numArgs) {
		  case 0:
			$this->_emptyUser();
			break;
		  case 1:
			$arg1 = func_get_arg(0);
			if (is_array($arg1)) {
				$ret = $this->_createUser($arg1);
				if (! $ret) {
					$this = '';
				}
			} else {
				$this->_fetchUser($arg1);
			}
			break;
		  case 2:
			$arg1 = func_get_arg(0); // uid or mail address
			$arg2 = func_get_arg(2); // array of attributes
			$this->_fetchUser($arg1, $arg2);
		  case 3:
			$arg1 = func_get_arg(0);
			$arg2 = func_get_arg(1);
			$arg3 = func_get_arg(2);
			$this->User3($arg1, $arg2, $arg3);
			break;
		  default:
			$this = '';
			break;
		}

	}

	function _fetchUser($uid, $argArray = '') {

		if ($uid == '') {
			$this = '';
			return;
		}

		$attrs = '--all';
		$cmd = new Command('msguserlist');
		if (strpos($uid, '@') === false) {
			$cmd->add_arg('--uid', $uid);
		} else {
			$cmd->add_arg('--mail', $uid);
		}
		$cmd->add_arg($attrs);

		$cmd->run();

		if ($cmd->getExitVal() !== 0) {
			$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
			$GLOBALS['LAST_ERROR'][] = getIntlMsg('ERR_CMD_ON', $cmd->concat());
			$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
			$this = '';
			return;
		}

		$lines = $cmd->getOutput();

		if ($lines == '') {
			$this = '';
			return;
		}

		$count = 0;
		$lines = array_reverse($lines);
		while ($lines) {
			$line = trim(array_pop($lines));

			if ($line != '') {
				list($attr, $value) = split(':', $line, 2);

				switch ($attr) {
					case 'uid' :
						$count++;
						$this->uid = strtolower($value);
						break;
					case 'firstname' :
						$count++;
						$this->firstname = $value;
						break;
					case 'lastname' :
						$count++;
						$this->lastname = $value;
						break;
					case 'fullname' :
						$count++;
						$this->fullname = $value;
						break;
					case 'mail' :
						$count++;
						$this->mail = $value;
						$this->shortmail = $this->shortmail();
						break;
					case 'businessphone' :
						$count++;
						$this->wphone = $value;
						break;
					case 'mobile' :
						$count++;
						$this->mphone = $value;
						break;
					case 'homephone' :
						$count++;
						$this->hphone = $value;
						break;
					case 'pager' :
						$count++;
						$this->pager = $value;
						break;
					case 'fax' :
						$count++;
						$this->fax = $value;
						break;
					case 'title' :
						$count++;
						$this->title = $value;
						break;
					case 'mailstop' :
						$count++;
						$this->location = $value;
						break;
					case 'alternatemail' :
						$count++;
						$this->altmail = $value;
						break;
					case 'forward' :
						$count++;
						$this->forward = $value;
						break;
					case 'forwardcopy' :
						$count++;
						if (preg_match('/true/i', $value)) {
							$this->forwardcopy = true;
						} else {
							$this->forwardcopy = false;
						}
						break;
					case 'vacationenabled' :
						$count++;
						if (strcasecmp($value, 'true') == 0) {
							$this->vacation = true;
						} else {
							$this->vacation = false;
						}
						break;
					case 'vacationsubject' :
						$count++;
						$this->vsubject = $value;
						break;
					case 'vacationmessage' :
						$count++;
						while (true) {
						  if (substr($value, -1) == '\\') {
						    $this->vmessage .= substr($value, 0, -1) . "\n";
						    $value = array_pop($lines);
						  } else {
						    $this->vmessage .= $value;
						    break;
						  }
						}
						break;
					case 'spamfilterenabled' :
						$count++;
						if (strcasecmp($value, 'true') == 0) {
							$this->spamfilter = true;
						} else {
							$this->spamfilter = false;
						}
						break;
					case 'spamdestroyenabled' :
						$count++;
						if (preg_match('/true/i', $value)) {
							$this->spamdestroy = true;
						} else {
							$this->spamdestroy = false;
						}
						break;
					case 'spamallowed' :
						$count++;
						$this->spamallow[] = $value;
						break;
					case 'spamexclude' :
						$count++;
						$this->spamdeny[] = $value;
						break;
					case 'quotalimit' :
						$count++;
						$this->quota = $value;
						break;
					case 'quotaused' :
						$count++;
						$this->quotaused = $value;
						break;
					default:
						break;
				}
			}
		}

		if (! $count > 0 ) {
			$this = '';
			return;
		}
	}

	function _emptyUser() {
		; // nothing
	}

	function _createUser($varArray) {
		global $session_domain;

		// Can only be run by admin
		if (! isAdmin()) {
			$LAST_ERROR[] = getIntlMsg('ERR_AUTH_NO_CREATE_PERM');
			return false;
		}

		$cmd = new Command('msgusercreate');
		$cmd->add_arg('--domain', $session_domain);

		while (list ($attr, $val) = each ($varArray)) {

			$val = trim($val);

			if ($val == '')  continue;

			$attr = str_replace('_', '', $attr);

			if (! $this->_validate($attr, $val)) continue;

			switch ($attr) {
				case 'uid' :
				    $val = strtolower($val);
				    $this->uid = $val;
				    break;
				case 'mail' :
				case 'altmail' :
				    if ($val != '') {
				    	$val = $val. '@' . $session_domain;
				    }
				    break;
				case 'firstname' :
				    $this->firstname = $val;
				    break;
				case 'lastname' :
				    $this->lastname = $val;
				    break;
				case 'fullname' :
				    $this->fullname = $val;
				    break;
				default:
				    break;
			}

			$opt = $this->_getOption($attr);
			if ($opt != '') {
				$cmd->add_arg($opt, $val);
			}
		}

		if ($this->fullname === '') {
			$opt = $this->_getOption('fullname');
			$val = $this->firstname . ' ' . $this->lastname;
			if ($opt != '') {
				$cmd->add_arg($opt, $val);
			}
		}

		$cmd->run();

		if ($cmd->getExitVal() === 0) {
			$this->_fetchUser($this->uid);
			return true;
		}
		$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
		$GLOBALS['LAST_ERROR'][] = getIntlMsg('ERR_CMD_ON', $cmd->concat());
		$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
		return false;
	}

	function _validate($attr, $value) {
		return true;
	}

	function _getOption($attr) {
		$attr = str_replace('_', '', $attr);

		switch ($attr) {
			case 'uid' :
				$retval = '--uid';
				break;
			case 'lastname' :
				$retval = '--lastname';
				break;
			case 'firstname' :
				$retval = '--firstname';
				break;
			case 'fullname' :
				$retval = '--fullname';
				break;
			case 'mail' :
				$retval = '--mail';
				break;
			case 'hphone' :
				$retval = '--homephone';
				break;
			case 'pager' :
				$retval = '--pager';
				break;
			case 'mphone' :
				$retval = '--mobile';
				break;
			case 'wphone' :
				$retval = '--businessphone';
				break;
			case 'fax' :
				$retval = '--fax';
				break;
			case 'title' :
				$retval = '--title';
				break;
			case 'location' :
				$retval = '--mailstop';
				break;
			case 'altmail' :
				$retval = '--alternatemail';
				break;
			case 'forward' :
				$retval = '--forward';
				break;
			case 'forwardcopy' :
				$retval = '--forwardcopy';
				break;
			case 'vacation' :
				$retval = '--vacationenabled';
				break;
			case 'vsubject' :
				$retval = '--vacationsubject';
				break;
			case 'vmessage' :
				$retval = '--vacationmessage';
				break;
			case 'spamfilter' :
				$retval = '--spamfilterenabled';
				break;
			case 'spamdestroy' :
				$retval = '--spamdestroyenabled';
				break;
			case 'spamallow' :
			case 'allowfrom' :
				$retval = '--spamallowed';
				break;
			case 'spamdeny' :
			case 'denyfrom' :
				$retval = '--spamexclude';
				break;
			case 'quota' :
				$retval = '--quotalimit';
				break;
			case 'quotaused' :
				$retval = '';
				break;
			default:
				$retval = '';
		}

		return $retval;
	}

	function formExport() {
		$attrs = get_object_vars($this);
		foreach ($attrs as $attr => $val) {
			$GLOBALS["_$attr"] = $val;
		}
	}

	function setAttr($attr, $val) {
		switch ($attr) {
			case 'uid':
				$this->uid = $val;
				break;
			case 'fullname':
				$this->fullname = $val;
				break;
			case 'mail':
				$this->mail = $val;
				$this->shortmail = $this->shortmail();
				break;
			case 'quotalimit':
				$this->quota = $val;
				break;
			case 'quotaused':
				$this->quotaused = $val;
				break;
			default:
				break;
		}
	}

	function setUID($value) {
		$this->uid = $value;
	}

	function getUID() {
		if ($this->uid == '') return getIntlMsg('MSG_EMPTY');
		return $this->uid;
	}

	function setFullName($value) {
		$this->fullname = $value;
	}

	function getFullName() {
		if ($this->fullname == '') return getIntlMsg('MSG_EMPTY');
		return $this->fullname;
	}

	function setMail($value) {
		$this->mail = $value;
	}

	function getQuota() {
		if (is_numeric($this->quota)) {
			return kbToMb($this->quota);
		}
		return false;
	}

	function getQuotaUsed() {
		if (is_numeric($this->quota)) {
			return kbToMb($this->quotaused);
		}
		return false;
	}

	function getQuotaFree() {
		if (is_numeric($this->quota)) {
			$y = kbToMb($this->quota) - kbToMb($this->quotaused);
			return ($y > 0) ? $y : 0;
		}
		return false;
	}

	function getQuotaPercent() {
		if (is_numeric($this->quota) && $this->quota > 0) {
			$tmp = kbToMB($this->quotaused) / kbToMb($this->quota);
			$tmp *= 100;
			return round($tmp, 1) . getIntlMsg('MSG_PERCENT');
		}
		return false;
	}

	function getMail() {
		if ($this->mail == '') return getIntlMsg('MSG_EMPTY');
		return $this->mail;
	}

	function setidLabel($value) {
		$this->idLabel = $value;
	}

	function delete($uid) {
		if (Authenticate::isAdmin($uid)) {
			$GLOBALS['LAST_ERROR'][] = getIntlMsg('MSG_CAN_NOT_DELETE_ADMIN');
			return false;
		}
	
		$cmd = new Command('msguserdelete');
		$cmd->add_arg('--uid', $uid);
		$cmd->add_arg('--recurse');
		$cmd->run();

		if ($cmd->getExitVal() == 0) {
			return true;
		}

		$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
		$GLOBALS['LAST_ERROR'][] = getIntlMsg('ERR_CMD_ON', $cmd->concat());
		$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
		return false;
	}

	function comparisonFunctionName($field, $direction) {
		switch($field) {
			case 'uid':
				$cmpFunction = 'byUID';
				break;
			case 'name':
				$cmpFunction = 'byFullName';
				break;
			case 'mail':
				$cmpFunction = 'byMail';
				break;
			case 'quota':
				$cmpFunction = 'byQuota';
				break;
			case 'used':
			case 'quotaused':
				$cmpFunction = 'byQuotaUsed';
				break;
			case 'free':
				$cmpFunction = 'byQuotaFree';
				break;
			case 'percent':
				$cmpFunction = 'byQuotaPercent';
				break;
			default:
				$cmpFunction = 'byUID';
				break;
		}

		switch($direction) {
			case 'a':
				$cmpFunction .= 'Ascend';
				break;
			case 'd':
				$cmpFunction .= 'Decend';
				break;
			default:
				$cmpFunction .= 'Acend';
				break;
		}
		return $cmpFunction;
	}

	function match($expr, $attr='') {
		$expr = substr($expr, 0, 256);
		if ($attr != '') {
			$retval = @ereg($expr, $this->$attr);
		} else {
			if (	@eregi($expr, $this->uid)     ||
				@eregi($expr, $this->fullname)   ||
				@eregi($expr, $this->shortmail)   ) {
				$retval = true;
			} else {
				$retval = false;
			}
		}
		return $retval;
	}

	function setForward($varArray) {
		global $LAST_ERROR, $LAST_ERROR_EXIT, $session_user, $session_perms;

		// If not admin... can only modify self.
		// Ensure modify priv has been granted.
		if (! isAdmin()) {
			if ($this->uid != $session_user) {
				$LAST_ERROR[] = getIntlMsg('ERR_AUTH_NO_MODIFY_PERM');
				return false;
			}
			if (! $session_perms->canChangeForward()) {
				$LAST_ERROR[] = getIntlMsg('ERR_AUTH_NO_MODIFY_PERM');
				return false;
			}
		}

		if (! isset($varArray['_forward'])) {
			return false;
		}
		$cmd = new Command('msgusermod');
		$cmd->add_arg('--uid', $this->uid);
		$current = get_object_vars($this);

		$opt = $this->_getOption('_forward');
		$val = $varArray['_forward'];
		if ($opt != '') {
			$cmd->add_arg($opt, $val);
		}

		if (isset($varArray['_forwardcopy'])) {
			$opt = $this->_getOption('_forwardcopy');
			$val = $varArray['_forwardcopy'];
			if (preg_match('/true/i', $val)) {
				$val = 'TRUE';
			} else {
				$val = 'FALSE';
			}
			if ($opt != '') {
				$cmd->add_arg($opt, $val);
			}
		}

		$cmd->run();

		if ($cmd->getExitVal() == 0) {
			$this = new User($this->uid);
			return true;
		}
		$LAST_ERROR = $cmd->getErrors();
		$LAST_ERROR[] = getIntlMsg('ERR_CMD_ON', $cmd->concat());
		$LAST_ERROR[] = getIntlMsg('MSG_FORWARD_ADDREMOVE_FAILURE');
		$LAST_ERROR_EXIT = $cmd->getExitVal();
		return false;
	}

	function setVacation($varArray) {
		global $LAST_ERROR, $LAST_ERROR_EXIT, $session_user, $session_perms;

		// If not admin... can only modify self.
		// Ensure modify priv has been granted.
		if (! isAdmin()) {
			if ($this->uid != $session_user) {
				$LAST_ERROR[] = getIntlMsg('ERR_AUTH_NO_MODIFY_PERM');
				return false;
			}
			if (! $session_perms->canChangeVacation()) {
				$LAST_ERROR[] = getIntlMsg('ERR_AUTH_NO_MODIFY_PERM');
				return false;
			}
		}

		#if (! isset($varArray['_forward'])) {
		#	return false;
		#}
		$cmd = new Command('msgusermod');
		$cmd->add_arg('--uid', $this->uid);
		$current = get_object_vars($this);

		$opt = $this->_getOption('_vacation');
		$val = $varArray['_vacation'];
		if (preg_match('/true/i', $val)) {
			$val = 'TRUE';
			$tmpval = true;
		} else {
			$val = 'FALSE';
			$tmpval = false;
		}
		if ($tmpval != $this->vacation && $opt != '') {
			$cmd->add_arg($opt, $val);
		}

		$opt = $this->_getOption('_vsubject');
		$val = $varArray['_vsubject'];
		if ($val != $this->vsubject && $opt != '') {
			$cmd->add_arg($opt, $val);
		}

		$opt = $this->_getOption('_vmessage');
		$val = $varArray['_vmessage'];
		$val = ereg_replace("\r", '', $val);
		if ($val != $this->vmessage && $opt != '') {
			$cmd->add_arg($opt, $val);
		}

		$cmd->run();

		if ($cmd->getExitVal() == 0) {
			$this = new User($this->uid);
			return true;
		}
		$LAST_ERROR = $cmd->getErrors();
		$LAST_ERROR[] = getIntlMsg('ERR_CMD_ON', $cmd->concat());
		$LAST_ERROR[] = getIntlMsg('MSG_FORWARD_ADDREMOVE_FAILURE');
		$LAST_ERROR_EXIT = $cmd->getExitVal();
		return false;
	}

	function setSpamFilter($varArray) {
		global $LAST_ERROR, $LAST_ERROR_EXIT, $session_user, $session_perms;

		// If not admin... can only modify self.
		// Ensure modify priv has been granted.
		if (! isAdmin()) {
			if ($this->uid != $session_user) {
				$LAST_ERROR[] = getIntlMsg('ERR_AUTH_NO_MODIFY_PERM');
				return false;
			}
			if (! $session_perms->canChangeJunkMail()) {
				$LAST_ERROR[] = getIntlMsg('ERR_AUTH_NO_MODIFY_PERM');
				return false;
			}
		}

		$cmd = new Command('msgusermod');
		$cmd->add_arg('--uid', $this->uid);
		$current = get_object_vars($this);

		while (list ($attr, $val) = each ($varArray)) {
			$skip = false;

			$val = trim($val);

			$attr = str_replace('_', '', $attr);

			if (! $this->_validate($attr, $val)) continue;

			if ($current[$attr] === $val) continue;
			switch ($attr) {
				case 'spamfilter':
				case 'spamdestroy':
				    //$val = strtoupper($val);
				    if (preg_match('/true/i', $val)) {
					$val = 'TRUE';
				    } else {
					$val = 'FALSE';
				    }
				    break;
				case 'spamallow':
				case 'spamdeny':
				    break;
				default: 
				    $skip = true;
				    break;
			}

			if (! $skip) {
				$opt = $this->_getOption($attr);
				if ($opt != '') {
					$cmd->add_arg($opt, $val);
				}
			}
		}

		$cmd->run();

		if ($cmd->getExitVal() == 0) {
			$this = new User($this->uid);
			return true;
		}
		$LAST_ERROR = $cmd->getErrors();
		$LAST_ERROR[] = getIntlMsg('ERR_CMD_ON', $cmd->concat());
		$LAST_ERROR[] = getIntlMsg('MSG_USERS_JUNKMAIL_MODIFY_FAILURE');
		$LAST_ERROR_EXIT = $cmd->getExitVal();
		return false;
	}

	function modify($varArray) {
		global $session_domain, $session_perms, $session_user, $LAST_ERROR, $LAST_ERROR_EXIT;

		// If not admin... can only modify self.
		// Ensure modify priv has been granted.
		if (! isAdmin()) {
			if ($this->uid != $session_user) {
				$LAST_ERROR[] = getIntlMsg('ERR_AUTH_NO_MODIFY_PERM');
				return false;
			}

			if (! $session_perms->canChangeProfile()) {
				$LAST_ERROR[] = getIntlMsg('ERR_AUTH_NO_MODIFY_PERM');
				return false;
			}
		}

		$cmd = new Command('msgusermod');
		$cmd->add_arg('--uid', $this->uid);
		$current = get_object_vars($this);

		while (list ($attr, $val) = each ($varArray)) {
			$skip = false;

			$val = trim($val);

			$attr = str_replace('_', '', $attr);

			if (! $this->_validate($attr, $val)) continue;

			if ($current[$attr] === $val) continue;
			switch ($attr) {
				case 'uid' :
				    continue; // uid cannot be modified
				case 'mail' :
				case 'altmail' :
				    if (! isAdmin()) break;

				    if ($val != '') {
				    	$val = $val. '@' . $session_domain;
				    }

				    if ($current[$attr] == $val) continue 2;

				    break;
				case 'forwardcopy':
				case 'vacation':
				case 'spamfilter':
				case 'spamdestroy':
				    //$val = strtoupper($val);
				    if (preg_match('/true/i', $val)) {
					$val = 'TRUE';
				    } else {
					$val = 'FALSE';
				    }
				    break;
				case 'vmessage':
				    $val = ereg_replace("\r", '', $val);
				    break;
				case 'quota':
				    if (! isAdmin()) {
					$LAST_ERROR[] = getIntlMsg('ERR_AUTH_NO_MODIFY_PERM');
					return false;
				    }
				    if ($val === '') {
					$val = 'NONE';
				    }
				    break;
				default: 
				    break;
			}

			if (! $skip) {
				$opt = $this->_getOption($attr);
				if ($opt != '') {
					$cmd->add_arg($opt, $val);
				}
			}
		}

		$cmd->run();

		if ($cmd->getExitVal() == 0) {
			$this = new User($this->uid);
			return true;
		}
		$LAST_ERROR = $cmd->getErrors();
		$LAST_ERROR[] = getIntlMsg('ERR_CMD_ON', $cmd->concat());
		$LAST_ERROR[] = getIntlMsg('MSG_PROFILE_MODIFY_FAILURE');
		$LAST_ERROR_EXIT = $cmd->getExitVal();
		return false;
	}

	function isRequired($attr) {
		switch ($attr) {
			case 'uid':
			case 'mail':
			case 'pw':
				$retval = true;
				break;
			case 'fullname':
			case 'firstname':
			case 'lastname':
			case 'wphone':
			case 'mphone':
			case 'hphone':
			case 'pager':
			case 'fax':
			case 'title':
			case 'location':
			case 'altmail':
			case 'forward':
			case 'forwardcopy':
			case 'vacation':
			case 'vsubject':
			case 'vmessage':
			case 'spamfilter':
			case 'spamdestroy':
			case 'spamallow':
			case 'spamdeny':
			default:
				$retval = false;
				break;
		}
		return $retval;
	}

	

	function display() {
		$retval  = $this->shortmail();
		$retval .= ' ' . getIntlMsg('MSG_PAREN_OPEN');
		$retval .= $this->fullname;
		$retval .= getIntlMsg('MSG_PAREN_CLOSE');

		return $retval;
	}

	function shortmail() {
		$pos = strpos($this->mail, '@');
		if ($pos === false) {
			$retval = $this->mail;
		} else {
			$retval = substr($this->mail, 0, $pos);
		}
		
		return $retval;
	}

	function shortaltmail() {
		$pos = strpos($this->altmail, '@');
		if ($pos === false) {
			$retval = $this->altmail;
		} else {
			$retval = substr($this->altmail, 0, $pos);
		}
		
		return $retval;
	}

	function addRule($rule) {
		global $session_perms;

		if (! $session_perms->canChangeProfile()) {
			$GLOBALS['LAST_ERROR'] = array(
				getIntlMsg('ERR_AUTH_NO_MODIFY_PERM'));
			return false;
		}

		if ($rule == '') {
			$GLOBALS['LAST_ERROR'] = array(
				getIntlMsg('ERR_RULES_NO_RULE_SPECIFIED'));
			return false;
		}


		switch ($rule->class) {
			case 'allowfrom':
			case 'denyfrom':
				$retval = $this->_addAllowDenyRule($rule->id, $rule->class);
				break;
			default:
				$GLOBALS['LAST_ERROR'] = array(getIntlMsg('ERR_RULES_UNSUPPORTED_CLASS', $rule->class));
				$retval = false;
				break;
                }
 
                return $retval;
	}


	function _addAllowDenyRule($s, $class) {

		// Make sure the string is unique in both allow and deny
		// arrays
		if (	(isset($this->spamallow)) &&
			(in_array($s, $this->spamallow))) {

                        $GLOBALS['LAST_ERROR'] = array(
                                getIntlMsg('ERR_RULES_DUPLICATE_RULE', $s,
				getIntlMsg('MSG_USERS_RULE_CLASS_ALLOW')));
                        return false;
		}
		if (	(isset($this->spamdeny)) &&
			(in_array($s, $this->spamdeny))) {

                        $GLOBALS['LAST_ERROR'] = array(
                                getIntlMsg('ERR_RULES_DUPLICATE_RULE', $s,
				getIntlMsg('MSG_USERS_RULE_CLASS_DENY')));
                        return false;
		}

		$cmd = new Command('msgusermod');
		$cmd->add_arg('--uid', $this->uid);
		$cmd->add_arg($this->_getOption($class), $s);

		$cmd->run();

		if ($cmd->getExitVal() == 0) {
			return true;
		}
		$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
		$GLOBALS['LAST_ERROR'][] = getIntlMsg('ERR_CMD_ON', $cmd->concat());
		$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
		return false;

	}

	function removeRule($rule) {
		global $session_perms;

		if (! $session_perms->canChangeProfile()) {
			$GLOBALS['LAST_ERROR'] = array(
				getIntlMsg('ERR_AUTH_NO_MODIFY_PERM'));
			return false;
		}

		if ($rule == '') {
			$GLOBALS['LAST_ERROR'] = array(
				getIntlMsg('ERR_RULES_NO_RULE_SPECIFIED'));
			return false;
		}
	
		switch ($rule->class) {
			case 'allowfrom':
				$retval = $this->_removeAllowRule($rule->id);
				break;
			case 'denyfrom':
				$retval = $this->_removeDenyRule($rule->id);
				break;
			default:
				$GLOBALS['LAST_ERROR'] = array(getIntlMsg('ERR_RULES_UNSUPPORTED_CLASS', $rule->class));
				$retval = false;
				break;
		}

		return $retval;
	}

	function _removeAllowRule($s) {

		if (	(! isset($this->spamallow)) ||
			(! in_array($s, $this->spamallow))) {

			$GLOBALS['LAST_ERROR'] = array(
				getIntlMsg('ERR_RULES_NO_SUCH_ENTRY', $s));
			return false;
		}

		$cmd = new Command('msgusermod');
		$cmd->add_arg('--uid', $this->uid);
		$cmd->add_arg('--delallowed', $s);

		$cmd->run();

		if ($cmd->getExitVal() == 0) {
			return true;
		}
		$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
		$GLOBALS['LAST_ERROR'][] = getIntlMsg('ERR_CMD_ON', $cmd->concat());
		$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
		return false;
	}

	function _removeDenyRule($s) {

		if (    (! isset($this->spamdeny)) || 
			(! in_array($s, $this->spamdeny))) {

			$GLOBALS['LAST_ERROR'] = array(
				getIntlMsg('ERR_RULES_NO_SUCH_ENTRY', $s));
			return false;

		}

		$cmd = new Command('msgusermod');
		$cmd->add_arg('--uid', $this->uid);
		$cmd->add_arg('--delexclude', $s);

		$cmd->run();

		if ($cmd->getExitVal() == 0) {
			return true;
		}
		$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
		$GLOBALS['LAST_ERROR'][] = getIntlMsg('ERR_CMD_ON', $cmd->concat());
		$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
		return false;
	}

}

/*
 * Ideally these comparison functions would be encapsulated in the 
 * User class as class methods. Can't work out how to pass a class
 * method to the usort() function yet.
 */
function byUIDAscend($left, $right) {
	return(strcasecmp($left->uid,$right->uid));
}

function byUIDDecend($left, $right) {
	return(-1 * (byUIDAscend($left, $right)));
}

function byFullNameAscend($left, $right) {
	return(strcasecmp($left->fullname,$right->fullname));
}

function byFullNameDecend($left, $right) {
	return(-1 * (byFullNameAscend($left, $right)));
}

function byMailAscend($left, $right) {
	return(strcasecmp($left->mail,$right->mail));
}

function byMailDecend($left, $right) {
	return(-1 * (byMailAscend($left, $right)));
}

function byQuotaAscend($left, $right) {
	return(($left->getQuota() > $right->getQuota()) ? 1 : -1);
}

function byQuotaDecend($left, $right) {
	return(-1 * (byQuotaAscend($left, $right)));
}

function byQuotaUsedAscend($left, $right) {
	return(($left->getQuotaUsed() > $right->getQuotaUsed()) ? 1 : -1);
}

function byQuotaUsedDecend($left, $right) {
	return(-1 * (byQuotaUsedAscend($left, $right)));
}

function byQuotaFreeAscend($left, $right) {
	return(($left->getQuotaFree() > $right->getQuotaFree()) ? 1 : -1);
}

function byQuotaFreeDecend($left, $right) {
	return(-1 * (byQuotaFreeAscend($left, $right)));
}

function byQuotaPercentAscend($left, $right) {
	return(($left->getQuotaPercent() > $right->getQuotaPercent()) ? 1 : -1);
}

function byQuotaPercentDecend($left, $right) {
	return(-1 * (byQuotaPercentAscend($left, $right)));
}

?>
