#!/bin/sh

vchk() \
{
	basename=$1
	fullname=`basename $2`

	./installation/vchk.pl $basename $fullname > /dev/null 2>&1
	ret=$?
	return $ret
}
ckremove()
{
	testvar=$1
	package=$2
	if [ $testvar = "TRUE" ]; then
		echo "Removing $2" | tee -a $LOGFILE
		rpm -e $package 2>&1 | tee -a $LOGFILE
	fi
}

LOGFILE=/var/tmp/msginstall.log
RESOURCEDIR="./installation/resources"

echo "Starting Installation of SCOoffice Mail Server..." | tee -a $LOGFILE

msgIsInstalled="FALSE"
rpm -q scomsg && msgIsInstalled="TRUE"

msgNewIsInstalled="FALSE"
rpm -q scomsg-2.0.0 && msgNewIsInstalled="TRUE"

onOpenUnix="FALSE"
[ -f /unixware/etc/rc0.d/K02audit ] && onOpenUnix="TRUE"

mtabaseInstalled="FALSE"
rpm -q mtabase && mtabaseInstalled="TRUE"

sendmailCfInstalled="FALSE"
rpm -q sendmail-cf && sendmailCfInstalled="TRUE"

sendmailDocInstalled="FALSE"
rpm -q sendmail-doc && sendmailDocInstalled="TRUE"

sendmailInstalled="FALSE"
rpm -q sendmail && sendmailInstalled="TRUE"

imapDevelInstalled="FALSE"
rpm -q imap-devel && imapDevelInstalled="TRUE"

imapInstalled="FALSE"
rpm -q imap && imapInstalled="TRUE"

postfixInstalled="FALSE"
rpm -q postfix && postfixInstalled="TRUE"

cyrusInstalled="FALSE"
rpm -q cyrus && cyrusInstalled="TRUE"

cyrusSASLInstalled="FALSE"
rpm -q cyrus-sasl  && cyrusSASLInstalled="TRUE"
rpm -q cyrus-sasl2 && cyrusSASLInstalled="TRUE"

libtoolInstalled="FALSE"
rpm -q libtool && libtoolInstalled="TRUE"


hordeInstalled="FALSE"
vchk horde ./OpenLinux/RPMS/horde-* && hordeInstalled="TRUE"


impInstalled="FALSE"
vchk imp ./OpenLinux/RPMS/imp-* && impInstalled="TRUE"

sendmailRunning="FALSE"
ps -elf | grep sendmail | grep -v grep > /dev/null && sendmailRunning="TRUE"

postfixRunning="FALSE"
ps -elf | grep postfix | grep -v grep > /dev/null && postfixRunning="TRUE"

cyrusRunning="FALSE"
ps -elf | grep cyrus | grep -v grep > /dev/null && cyrusRunning="TRUE"

cyrusSASLRunning="FALSE"
ps -elf | grep sasl | grep -v grep > /dev/null && cyrusSASLRunning="TRUE"

ldapRunning="FALSE"
ps -elf | grep slapd | grep -v grep > /dev/null && ldapRunning="TRUE"

preinstallOkay="FALSE"
$RESOURCEDIR/preinstall2.sh /tmp/preinstall.log && preinstallOkay="TRUE"


ipu="FALSE"
rpm -q volutionmsg && ipu="TRUE"

checkPerlMsgcat="FALSE"
perl -mLocale::Msgcat -e print && checkPerlMsgcat="TRUE"

if [ $msgIsInstalled = "TRUE" -a $msgNewIsInstalled = "TRUE" ]; then
	echo "SCOoffice Mail Server 2.0.0 Is Already Installed" >&2 | tee -a $LOGFILE
	exit 1
fi

if [ $ipu = "TRUE" ]; then
	$RESOURCEDIR/preinstall-ipu.sh | tee -a $LOGFILE
fi

if [ $onOpenUnix = "TRUE" ]; then
	$RESOURCEDIR/lkp_master lkp_msg
fi

if [ $sendmailRunning = "TRUE" ]; then
	/etc/rc.d/init.d/mta stop
fi

if [ $postfixRunning = "TRUE" ]; then
	/etc/rc.d/init.d/postfix stop
fi

if [ $cyrusRunning = "TRUE" ]; then
	/etc/rc.d/init.d/cyrus stop
fi

if [ $cyrusSASLRunning = "TRUE" ]; then
	/etc/rc.d/init.d/saslauthd stop
	killall -9 saslauthd
fi

if [ $ldapRunning="TRUE" ]; then
	/etc/rc.d/init.d/ldap stop
fi

if [ $mtabaseInstalled = "TRUE" ]; then
	echo "Removing mtabase" | tee -a $LOGFILE
	rpm -e --nodeps mtabase  2>&1 | tee -a $LOGFILE
fi

ckremove $sendmailCFInstalled sendmail-cf
ckremove $sendmailDocInstalled sendmail-doc
ckremove $sendmailInstalled sendmail
ckremove $imapDevelInstalled imap-devel
ckremove $imapInstalled imap

vchk db ./OpenLinux/RPMS/db-* && {
	echo "Installing Sleepycat DB"  | tee -a $LOGFILE
	rpm -Uh ./OpenLinux/RPMS/db-* 2>&1 | tee -a $LOGFILE
}
vchk postfix ./OpenLinux/RPMS/postfix-* && {
	echo "Installing Postfix"| tee -a $LOGFILE
	rpm -Uh --force ./OpenLinux/RPMS/postfix-* 2>&1 | tee -a $LOGFILE
}
vchk libtool ./OpenLinux/RPMS/libtool-* && {
	echo "Installing Libtool"| tee -a $LOGFILE
	rpm -Uh --nodeps ./OpenLinux/RPMS/libtool-* 2>&1 | tee -a $LOGFILE
}
vchk drac ./OpenLinux/RPMS/drac-* && {
	echo "Installing Dynamic Authorization Relay Control"| tee -a $LOGFILE
	rpm -Uh ./OpenLinux/RPMS/drac-* 2>&1 | tee -a $LOGFILE
}
vchk libmcrypt ./OpenLinux/RPMS/libmcrypt-* && {
	echo "Installing MCrypt Library"| tee -a $LOGFILE
	rpm -Uh ./OpenLinux/RPMS/libmcrypt-* 2>&1 | tee -a $LOGFILE
}
vchk msg-doc ./OpenLinux/RPMS/scomsg-doc-* && {
	echo "Installing SCOoffice Mail Server Documentation"| tee -a $LOGFILE
	rpm -Uh ./OpenLinux/RPMS/scomsg-doc-* 2>&1 | tee -a $LOGFILE
}
vchk cyrus-sasl2 ./OpenLinux/RPMS/cyrus-sasl2-* && {
	echo "Installing Cyrus SASL"| tee -a $LOGFILE
	rpm -Uh --nodeps --force ./OpenLinux/RPMS/cyrus-sasl2-* 
}
vchk cyrus-imapd ./OpenLinux/RPMS/cyrus-imapd-* && {
	echo "Installing Cyrus IMAPD"| tee -a $LOGFILE
	rpm -Uh --nodeps --force ./OpenLinux/RPMS/cyrus-imapd-*
}
vchk gq ./OpenLinux/RPMS/gq-* && {
	echo "Installing GQ"| tee -a $LOGFILE
	rpm -Uh ./OpenLinux/RPMS/gq-* 2>&1 | tee -a $LOGFILE
}
vchk perl-Convert-ASN1 ./OpenLinux/RPMS/perl-Convert-ASN1-* && {
	echo "Installing Perl Convert-ASN1"| tee -a $LOGFILE
	rpm -Uh ./OpenLinux/RPMS/perl-Convert-ASN1-* 2>&1 | tee -a $LOGFILE
}
vchk perl-IMAP-Admin ./OpenLinux/RPMS/perl-IMAP-Admin-* && {
	echo "Installing Perl IMAP-Admin"| tee -a $LOGFILE
	rpm -Uh ./OpenLinux/RPMS/perl-IMAP-Admin-* 2>&1 | tee -a $LOGFILE
}
vchk perl-Mcrypt ./OpenLinux/RPMS/perl-Mcrypt-* && {
	echo "Installing Perl MCrypt"| tee -a $LOGFILE
	rpm -Uh --nodeps ./OpenLinux/RPMS/perl-Mcrypt-* 2>&1 | tee -a $LOGFILE
}
vchk perl-ldap ./OpenLinux/RPMS/perl-ldap-* && {
	echo "Installing Perl LDAP"| tee -a $LOGFILE
	rpm -Uh ./OpenLinux/RPMS/perl-ldap-* 2>&1 | tee -a $LOGFILE
}
vchk perl-Text-Iconv ./OpenLinux/RPMS/perl-Text-Iconv-* && {
	echo "Installing Perl Text-Iconv"| tee -a $LOGFILE
	rpm -Uh ./OpenLinux/RPMS/perl-Text-Iconv-* 2>&1 | tee -a $LOGFILE
}
if [ $checkPerlMsgcat = "FALSE" ] ;then
	echo "Installing Perl Msgcat"| tee -a $LOGFILE
	rpm -Uh --nodeps ./OpenLinux/RPMS/perl-Msgcat-* 2>&1 | tee -a $LOGFILE
fi
for ispell in ispell-english ispell-french ispell-german ispell-italian ispell-polish ispell-spanish
do
	vchk $ispell ./OpenLinux/RPMS/$ispell-* && {
		echo "Installing $ispell"| tee -a $LOGFILE
		rpm -Uh ./OpenLinux/RPMS/$ispell-* 2>&1 | tee -a $LOGFILE
	}
done

vchk horde ./OpenLinux/RPMS/horde-* && {
	echo "Installing Horde Framework"| tee -a $LOGFILE
	rpm -Uh ./OpenLinux/RPMS/horde-* 2>&1 | tee -a $LOGFILE
}
vchk imp ./OpenLinux/RPMS/imp-* && {
	echo "Installing Web-based IMAP mail client"| tee -a $LOGFILE
	rpm -Uh ./OpenLinux/RPMS/imp-* 2>&1 | tee -a $LOGFILE
}
vchk scomsg ./OpenLinux/RPMS/scomsg-2.0.0 && {
	echo "Installing SCOoffice Mail Server"| tee -a $LOGFILE
	rpm -Uh ./OpenLinux/RPMS/scomsg-2.0.0-* 2>&1 | tee -a $LOGFILE
}

echo "Excuting Postinstall script"| tee -a $LOGFILE
$RESOURCEDIR/postinstall.sh > /var/tmp/postinstall.log 2>&1
echo "Done"| tee -a $LOGFILE

#vchk scomsg ./OpenLinux/RPMS/scomsg-*
