#!/usr/bin/perl
#
# Copyright (c) 2001-2002 Caldera International, Inc. All Rights Reserved.
#

use strict;

my $MSG_LDAPDIR=$ENV{MSG_LDAPDIR};
my $MSG_UNITEDLINUX=$ENV{MSG_UNITEDLINUX};
my $MSG_UPGRADE=$ENV{MSG_UPGRADE};
my $SLAPDCONF="$MSG_LDAPDIR/slapd.conf";
my $TMPFILE = "/var/tmp/slapd.conf";

my @includes;    # array of schemas to include as found in slapd.conf
my $pidfile;     # pidfile to use from slapd.conf
my $argsfile;    # arguments file to use from slapd.conf
my $disallow;    # dissallow directive from slapd.conf
my $passwdhash;  # passwordhash from slapd.conf
my $database;    # database from slapd.conf **hardcoded to ldbm**
my $suffix;      # base root suffix from slapd.conf
my $rootdn;      # root user for ldap from slapd.conf
my $rootpw;      # ldap root passwd from slapd.conf
my $directory;   # directory containing the ldap server from slapd.conf
my $sizelimit;   # size limit for directory entries
my @indexes;     # database indexes from slapd.conf
my $accessParam; # the current access directive being acted on
my @acls;        # the access directives from slapd.conf
my %byusers;     # 'by users' directives for access directives
my %bygroup;     # 'by group' directives for access directives
my %byself;      # 'by self' directives  for access directives
my %bydn;        # 'by dn' directives for access directives
my %bydnattr;    # 'by dnattr' directives for access directives
my %byanon;      # 'by anonymous' directives for access directives

# TLS certificates variables from slapd.conf
my $TLSCertificateFile; 
my $TLSCertificateKeyFile;
my $TLSCACertificateFile;

# Is this an upgrade or a fresh install of MSG Server?
my $isupgrade;

# This subroutine will be used for upgrades.
sub upgrade {
	# for now all we do is copy the system file to the config location
        # so it just gets reinstalled.  We do have to use a new password
        # if we can't grab a plain-text one from the file.our exist
	open(IFD, "$SLAPDCONF");
	open(OFD, ">$TMPFILE");
	foreach (<IFD>) {
		if (/^rootpw\s*\{/) {
			print OFD "rootpw\t\t".'%ROOTDNPW%'."\n";
		} else {
			print OFD "$_";
		}
	}
	close(IFD);
	close(OFD);
}

# This subroutine is used to parse the existing slapd.conf file and
# obtain any existing configurations.
sub parsedap {

	my @contents;

        open(FD, "$SLAPDCONF") || die "could not open slapd.conf";
	@contents = <FD>;
	close(FD);

	foreach (@contents) {
		chomp($_);
		if (/^include/ ) {
			push(@includes, $_);
			if(/$MSG_LDAPDIR\/schema\/msg.schema/) {
				$isupgrade = 1;
			}
		}
		elsif (/^index/ ) {
			push(@indexes, $_);
		}
		elsif (/^sizelimit/ ) {
			$sizelimit = $_;
		}
		elsif (/^pidfile/ ) {
			$pidfile = $_;
		}
		elsif (/^argsfile/) {
			$argsfile = $_;
		}
		elsif (/^disallow/) {
			$disallow = $_;
		}
		elsif (/^passwd-hash/) {
			$passwdhash = $_;
		}
		elsif (/^database/) {
			#$database = $_;
			$database = "database\tldbm";
		}
		elsif (/^suffix/) {
			$suffix = $_;
		}
		elsif (/^rootdn/) {
			$rootdn = $_;
		}
		elsif (/^rootpw/) {
			$rootpw = $_;
		}
		elsif (/^directory/) {
			$directory = $_;
		}
		elsif (/^access/) {
			$accessParam = $_;
			push(@acls, $accessParam);
		}
		elsif (/^\s*by dn=/) {
			push(@{$bydn{$accessParam}}, $_);
		}
		elsif (/^\s*by dnattr=/) {
			push(@{$bydnattr{$accessParam}}, $_);
		}
		elsif (/^\s*by users/) {
			$byusers{$accessParam} = $_;
		}
		elsif (/^\s*by group/) {
			$bygroup{$accessParam} = $_;
		}
		elsif (/^\s*by self/) {
			$byself{$accessParam} = $_;
		}
		elsif (/^\s*by anonymous/) {
			$byanon{$accessParam} = $_;
		}
		elsif (/^TLSCertificateFile/) {
			$TLSCertificateFile = $_;
			$TLSCertificateFile =~ s/TLSCertificateFile[ \t]*//;
		}
		elsif (/^TLSCertificateKeyFile/) {
			$TLSCertificateKeyFile = $_;
			$TLSCertificateKeyFile =~ s/TLSCertificateKeyFile[ \t]*//;
		}
		elsif (/^TLSCACertificateFile/) {
			$TLSCACertificateFile = $_;
			$TLSCACertificateFile =~ s/TLSCACertificateFile[ \t]*//;
		}
	}
}


###### MAIN #######

# Get the existing LDAP information
parsedap;

# Ok now we're ready to start.
# Are we just upgrading an existing messaging server?
# If so, we just call the upgrade proc.
if ($isupgrade) {
	upgrade();
	exit(0);
}

# 
# In this case, assume the old file hasn't been touched since 
# the default password is still there.
$_ = $rootpw;
if ($MSG_UPGRADE) {
	; # do nothing
} elsif ( /^rootpw\s*secret$/) {
	$rootdn = 'rootdn		%ROOTDNBIND%';
	$rootpw = 'rootpw		%ROOTDNPW%';
	$suffix = 'suffix		%ROOT%';
} elsif ( /^rootpw\s*{/) {
	$rootpw = 'rootpw		%ROOTDNPW%';
}


open(FD, ">$TMPFILE") || die "slapd.conf.msg:$!\n";
print FD '# See slapd.conf(5) for details on configuration options'."\n";
print FD '# This file should NOT be world readable'."\n";

my $gotcore=0;   # Keep track of if we found the core schema
my $gotcosine=0; # same for cosine schema
my $gotinet=0;   # and for inetorgperson schema

my $gotPasswd=0; # did we find a password access ACL
my $gotSplat=0;  # did we find a default '*' access ACL

foreach(@includes) {
	print FD "$_\n";
	if(/core\.schema/) {
		$gotcore = 1 ;
	} elsif (/cosine\.schema/) {
		$gotcosine = 1 ;
	} elsif (/inetorgperson\.schema/) {
		$gotinet = 1;
	} 
}
if(!$gotcore) {
	print FD "include         $MSG_LDAPDIR/schema/core.schema\n";
}
if(!$gotcosine) {
	print FD "include         $MSG_LDAPDIR/schema/cosine.schema\n";
}
if(!$gotinet) {
	print FD "include         $MSG_LDAPDIR/schema/inetorgperson.schema\n";
}
print FD "include         $MSG_LDAPDIR/schema/msg.schema\n";

print FD "\n";
print FD "$pidfile\n";
print FD "$argsfile\n";
print FD "\n";
#print FD "disallow        bind_anon_dn\n";
if ($MSG_UNITEDLINUX) {
	print FD "allow        bind_v2\n";
}
print FD "password-hash   {CRYPT}\n";
print FD "\n";
print FD '#####################################################'."\n";
print FD '# ldbm database definitions'."\n";
print FD '#####################################################'."\n";
print FD "$database\n";
print FD "$suffix\n";
print FD "\n";
print FD '#####################################################'."\n";
print FD '# Cleartext passwords, especially for the rootdn, should
# be avoided.  See slappasswd(8) and slapd.conf(5) for details.
# Use of strong authentication encouraged.'."\n";
print FD '#####################################################'."\n";
print FD "$rootdn\n";
print FD "$rootpw\n";
print FD "\n\n";
print FD '# The database directory MUST exist prior to running slapd AND
# should only be accessable by the slapd/tools. Mode 700 recommended.'."\n";
print FD "$directory\n";


# check for likely-conflicting ACL's
foreach(@acls) {
	if (/access to attr=userpassword/) {
		$gotPasswd = $_ ;
	}
	elsif (/access to \*/) {
		$gotSplat = $_ ;
	}
}

# Do the password ACL first.
print FD "\n";
if($gotPasswd) {
	$_ = $gotPasswd;
	print FD $_."\n";
	print FD '        by group.exact="cn=MSG Administrators,%BASE%" write'."\n";
	if($byanon{$_}) {
		print FD $byanon{$_}."\n";
	} else {
		print FD "        by anonymous auth\n";
	}
	print FD '        by self =wx'."\n";
	if($bydn{$_}) {
		foreach(@{$bydn{$_}}) {
			print FD $_."\n";
		}
	}
	if($bydnattr{$_}) {
		foreach(@{$bydnattr{$_}}) {
			print FD $_."\n";
		}
	}
	if($byusers{$_}) {
		print FD $byusers{$_}."\n";
	} else {
		print FD "        by users none\n";
	}
} else {
       	print FD 'access to attr=userpassword
	by group.exact="cn=MSG Administrators,%BASE%" write
	by self =wx
	by users none
	by anonymous auth'."\n";
}

# Messaging server-specific ACL's

# User attributes only writeable by admin
print FD "\n";
print FD 'access to attr=uid,ou,objectClass,mail,calderaMsgAlternateAddress,calderaMsgUserHost
	by group.exact="cn=MSG Administrators,%BASE%" write
	by users read
	by anonymous read'."\n";

print FD "\n";
print FD 'access to attr=calderaMsgQuotaLimit
	by group.exact="cn=MSG Administrators,%BASE%" write
	by dn="uid=msguser,.*" read
	by self read
	by users none
	by anonymous none'."\n";

# UserForwardWrite ACL
print FD "\n";
print FD 'access to attr=calderaMsgForwardingAddress,calderaMsgForwardCopy,calderaMsgDrop
	by group.exact="cn=MSG Administrators,%BASE%" write
	by dn="uid=msguser,.*" read
	by self write
	by users none
	by anonymous none'."\n";

# UserVactionWrite ACL
print FD "\n";
print FD 'access to attr=calderaMsgVacationEnabled,calderaMsgVacationSubject,calderaMsgVacationMessage
	by group.exact="cn=MSG Administrators,%BASE%" write
	by dn="uid=msguser,.*" read
	by self write
	by users none
	by anonymous none'."\n";

# UserSpamWrite ACL
print FD "\n";
print FD 'access to attr=calderaMsgSpamFilterEnabled,calderaMsgSpamAllowed,calderaMsgSpamExclude,calderaMsgSpamDestroyEnabled
	by group.exact="cn=MSG Administrators,%BASE%" write
	by dn="uid=msguser,.*" read
	by self write
	by users none
	by anonymous none'."\n";

# UserProfileWrite ACL - attrs we use
print FD "\n";
print FD 'access to attr=sn,cn,telephoneNumber,title,facsimileTelephoneNumber,physicalDeliveryOfficeName,displayName,givenName,homePhone,mobile,pager
	by group.exact="cn=MSG Administrators,%BASE%" write
	by self write
	by users read
	by anonymous read'."\n";

# UserProfileWrite ACL - attrs we don't use
print FD "\n";
print FD 'access to attr=seeAlso,description,x121Address,registeredAddress,destinationIndicator,preferredDeliveryMethod,telexNumber,teletexTerminalIdentifier,internationaliSDNNumber,street,postOfficeBox,postalCode,postalAddress,st,l,audio,businessCategory,carLicense,departmentNumber,employeeNumber,employeeType,homePostalAddress,initials,jpegPhoto,labeledURI,manager,o,photo,roomNumber,secretary,userCertificate,x500uniqueIdentifier,preferredLanguage,userSMIMECertificate,userPKCS12
	by group.exact="cn=MSG Administrators,%BASE%" write
	by self write
	by users read
	by anonymous read'."\n";

# This can restrict everything -- superceeds Forward,Vacation & Spam ACLs
print FD "\n";
print FD 'access to dn="uid=.*,ou=.*,ou=SCOoffice Server.*"
	by group.exact="cn=MSG Administrators,%BASE%" write
	by self write
	by users read
	by anonymous read'."\n";

# Alias Restrictions
print FD "\n";
print FD 'access to filter="(&(objectclass=msgGroup)(calderaMsgMgrpJoinable=TRUE))" attr=entry,member
	by group.exact="cn=MSG Administrators,%BASE%" write
	by dnattr=owner write
	by dnattr=member selfwrite
	by users read
	by anonymous read'."\n";

# These attributes aren't currently supported
#print FD "\n";
#print FD 'access to attr=calderaMsgMgrpDeliveryProgram
#	by group.exact="cn=MSG Administrators,%BASE%" write
#	by users read
#	by anonymous read'."\n";
#
#print FD "\n";
#print FD 'access to attr=calderaMsgMgrpDeliveryFile
#	by group.exact="cn=MSG Administrators,%BASE%" write
#	by users read
#	by anonymous read'."\n";

print FD "\n";
print FD 'access to attr=calderaMsgMgrpRFC822Member
	by group.exact="cn=MSG Administrators,%BASE%" write
	by dnattr=owner write
	by users read
	by anonymous read'."\n";

print FD "\n";
print FD 'access to filter="(objectclass=msgGroup)"
	by group.exact="cn=MSG Administrators,%BASE%" write
	by dnattr=owner write
	by users read
	by anonymous read'."\n";

print FD "\n";
print FD 'access to dn="ou=MSG Groups,ou=.*,%BASE%" attr=children
	by group.exact="cn=MSG Administrators,%BASE%" write
	by users write
	by anonymous read'."\n";

# ACL's already present in the old slapd.conf
foreach(@acls) {
	if (/access to attr=userpassword/) {
		next;
	}
	elsif (/access to \*/) {
		next;
	} else {
		print FD "\n";
		print FD $_."\n";
		if($byself{$_}) {
			print FD $byself{$_}."\n";
		}
		if($byanon{$_}) {
			print FD $byanon{$_}."\n";
		}
		if($bygroup{$_}) {
			print FD $bygroup{$_}."\n";
		}
		if($bydn{$_}) {
			foreach(@{$bydn{$_}}) {
				print FD $_."\n";
			}
		}
		if($bydnattr{$_}) {
			foreach(@{$bydnattr{$_}}) {
				print FD $_."\n";
			}
		}
		if($byusers{$_}) {
			print FD $byusers{$_}."\n";
		}
	}
}

# The dn.one ACL needed by the messaging server
print FD "\n";
print FD 'access to dn.one="%BASE%"
	by group.exact="cn=MSG Administrators,%BASE%" write
	by users read
	by anonymous read'."\n";

# The main dn ACL needed by the messaging server for anonymous access
print FD "\n";
print FD 'access to dn="%BASE%"
	by group.exact="cn=MSG Administrators,%BASE%" write
	by users read
	by anonymous read'."\n";

# The default ACL needed by the messaging server 
print FD "\n";
if($gotSplat) {
	$_ = $gotSplat;
	print FD $_."\n";
	print FD '        by group.exact="cn=MSG Administrators,%BASE%" write'."\n";
	if($byanon{$_}) {
		print FD $byanon{$_}."\n";
	} else {
		print FD "        by anonymous auth\n";
	}
	if($byself{$_}) {
		print FD $byself{$_}."\n";
	} 
	if($bygroup{$_}) {
		print FD $bygroup{$_}."\n";
	}
	print FD "        by users read\n";
	if($bydn{$_}) {
		foreach(@{$bydn{$_}}) {
			print FD $_."\n";
		}
	}
	if($bydnattr{$_}) {
		foreach(@{$bydnattr{$_}}) {
			print FD $_."\n";
		}
	}
} else {
       	print FD 'access to *
	by group.exact="cn=MSG Administrators,%BASE%" write
	by users read
	by anonymous auth'."\n";
}
print FD "\n";
print FD '# Indices to maintain'."\n";
print FD "\n";

foreach (@indexes) {
	if (/index\s*sn[\s|,]/) {
		next;
	} elsif (/index\s*given[\s|,]/) {
		next;
	} elsif (/index\s*mail[\s|,]/) {
		next;
	} elsif (/index\s*objectclass[\s|,]/) {
		next;
	} elsif (/index\s*objectClass[\s|,]/) {
		next;
	} elsif (/index\s*calderaMsgAlternateAddress[\s|,]/) {
		next;
	} elsif (/index\s*uid[\s|,]/) {
		next;
	} elsif (/index\s*member[\s|,]/) {
		next;
	} elsif (/index\s*owner[\s|,]/) {
		next;
	} elsif (/index\s*cn[\s|,]/) {
		next;
	} else {
		print FD $_."\n";
	}
}
print FD "index   sn,givenname                                     sub
index   mail                                              pres,sub,eq
index   objectclass,calderaMsgAlternateAddress,uid        eq
index   member,owner                                      pres,eq
index   cn                                                pres,eq\n";

# because ldap/mod_tls (run by init.d/ldap start)
# has no newlines in it's output which currupts the file so we do it right here.
if (!$TLSCertificateFile) {
	if ($MSG_UNITEDLINUX) {
		$TLSCertificateFile = "/etc/ssl/certs/hostcert.pem";
	} else {
		$TLSCertificateFile = "$MSG_LDAPDIR/certs/hostcert.pem";
	}
}
if (!$TLSCertificateKeyFile) {
	if ($MSG_UNITEDLINUX) {
		$TLSCertificateKeyFile = "/etc/ssl/private/hostkey.pem";
	} else {
		$TLSCertificateKeyFile = "$MSG_LDAPDIR/private/hostkey.pem";
	}
}
if (!$TLSCACertificateFile) {
	if ($MSG_UNITEDLINUX) {
		$TLSCACertificateFile = "/etc/ssl/certs/ca-cert.pem";
	} else {
		$TLSCACertificateFile = "$MSG_LDAPDIR/certs/cacerts.pem";
	}
}

print FD "
# If you would like to disable TLS/SSL support,
# please do not do it here.  Instead, use the
# SSLSUPPORT variable in /etc/rc.d/init.d/ldap

#TLSCertificateFile $TLSCertificateFile
#TLSCertificateKeyFile $TLSCertificateKeyFile\n";
if ($TLSCACertificateFile) {
	print FD "#TLSCACertificateFile $TLSCACertificateFile\n";
}

print FD "\n";
$_ = $sizelimit;

my @sizeinfo=split();

if ($sizeinfo[1] < 99999) { 
	print FD "sizelimit 99999\n";
} else {
	print FD "$sizelimit\n";
}

close(FD);
